package vp.graph;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.ListIterator;

/**
* The standard implementation of <code>DecorableElement</code>.
* It is used by implementations of <code>Edge</code> to maintain pertinent decorations.
*
* @see edu.yale.graph.DecorableElementImp
* @see edu.yale.graph.Edge
*/
public class DecorableElementImp implements DecorableElement
{
	private ArrayList decList;
	private ArrayList decListeners;

	/**
	* Creates a new decorable element with no decorations or decoration listeners.
	*/
	public DecorableElementImp()
	{
		decList = new ArrayList();
		decListeners = new ArrayList();
	}

	/**
	* Add the specified decoration to the list of decorations.
	*/
	public void addDecoration(Decoration decoration)
	{
		if (containsDecoration(decoration))
		{
			return;
		}

		GraphEvent event =
			new GraphEvent(this, GraphEvent.PRE_ADD_DEC, decoration);
		invokeDecorationListeners(event);

		decList.add(decoration);

		event = new GraphEvent(this, GraphEvent.AFT_ADD_DEC, decoration);
		invokeDecorationListeners(event);
	}

	/**
	* Add the specified decoration listener to the list of decoration listeners.
	*/
	public void addDecListener(DecorationListener listener)
	{
		decListeners.add(listener);
	}

	/**
	* Return true if the list of decorations contains the specified decoration, false otherwise.
	*/
	public boolean containsDecoration(Decoration decoration)
	{
		return decList.contains(decoration);
	}

	/**
	* Return the number of decorations in the list of decorations.
	*/
	public int getDecorationCount()
	{
		return decList.size();
	}

	/**
	* Return an iteration of the list of decorations.
	*/
	public ListIterator getDecorationIterator()
	{
		return getDecList().listIterator();
	}

	/**
	* Return the full list of decorations.
	*/
	public ArrayList getDecList()
	{
		return decList;
	}

	/**
	* Remove the specified decoration from the list of decorations.
	*/
	public void removeDecoration(Decoration decoration)
	{
		GraphEvent event =
			new GraphEvent(this, GraphEvent.PRE_DEL_DEC, decoration);
		invokeDecorationListeners(event);

		decList.remove(decoration);

		event = new GraphEvent(this, GraphEvent.AFT_DEL_DEC, decoration);
		invokeDecorationListeners(event);
	}

	/**
	* Remove all decorations from the list of decorations.
	*/
	public void removeAllDecorations()
	{
		ArrayList decsToRemove = new ArrayList();
		Iterator iter = getDecorationIterator();
		while (iter.hasNext())
		{
			Decoration dec = (Decoration)iter.next();
			decsToRemove.add(dec);
		}

		for (int i = 0; i < decsToRemove.size(); i++)
		{

			removeDecoration((Decoration)decsToRemove.get(i));
		}
	}

	/**
	* Remove the specified decoration listener from the list of decoration listeners.
	*/
	public void removeDecListener(DecorationListener listener)
	{
		decListeners.remove(listener);
	}

	private void invokeDecorationListeners(GraphEvent event)
	{
		int command = event.getCommand();
		Iterator iter = decListeners.iterator();
		while (iter.hasNext())
		{
			DecorationListener listener = (DecorationListener)iter.next();
			switch (command)
			{
				case GraphEvent.PRE_ADD_DEC :
					listener.beforeDecorationAdded(event);
					break;
				case GraphEvent.AFT_ADD_DEC :
					listener.afterDecorationAdded(event);
					break;
				case GraphEvent.PRE_DEL_DEC :
					listener.beforeDecorationRemoved(event);
					break;
				case GraphEvent.AFT_DEL_DEC :
					listener.afterDecorationRemoved(event);
					break;
				default :
					break;
			}
		}
	}
	/**
	 * @return
	 */
	public ArrayList getDecListeners()
	{
		return decListeners;
	}

	/**
	 * @param list
	 */
	public void setDecList(ArrayList list)
	{
		decList = list;
	}

	/**
	 * @param list
	 */
	public void setDecListeners(ArrayList list)
	{
		decListeners = list;
	}

	/* (non-Javadoc)
	 * @see vp.graph.DecorableElement#getDecoration(int)
	 */
	public Decoration getDecoration(int index)
	{
		return (Decoration)decList.get(index);
	}

}
