package vp.graph;

import java.util.List;
import java.util.ListIterator;

/**
* The central repository of information on the structure of the pathway graph.   
* It is used to create all representations of the graph, including the visual and XML versions.
* In so doing, the implementation maintains the following data sets:	<br>
* 1. A list of all edges and vertices in the pathway.	<br>
* 2. A list of all visual types that are either contained in the current graph or are defined for use.	<br>
* 3. A list of all compounds or enzymes not in the database that the user has added.	<br>
* 4. A list of all listeners associated with the graph.		<br>
* The standard implementation of <code>Graph</code> is <code>GraphImp</code>.  Its standard visual representation
* is <code>VisualGraph</code>.
*
* @see edu.yale.graph.GraphImp
* @see edu.yale.graph.visual.VisualGraph
*/
public interface Graph
{
	/**
	* Adds the specified edge to the graph.
	*/
	void addEdge(Edge edge);

	/**
	* Adds the specified vertex to the graph.
	*/
	void addVertex(Vertex vertex);

	/**
	* Returns true if the specified edge is in the pathway graph, false otherwise.
	*/
	boolean containsEdge(Edge edge);

	/**
	* Returns true if the specified vertex is in the pathway graph, false otherwise.
	*/
	boolean containsVertex(Vertex vertex);

	/**
	* Returns the total number of edges in the current pathway graph.
	*/
	int getEdgeCount();

	/**
	* Returns an iteration of all of the edges in the current pathway graph.
	*/
	ListIterator getEdgeIterator();

	/**
	* Returns a <code>HashSet</code> containing all of the edges in the current pathway graph.
	*/
	List getEdgeList();

	/**
	* Returns the total number of vertices in the current pathway graph.
	*/
	int getVertexCount();

	/**
	* Returns an iteration of all of the vertices in the current pathway graph.
	*/
	ListIterator getVertexIterator();

	/**
	* Returns a <code>HashSet</code> containing all of the vertices in the current pathway graph.
	*/
	List getVertexList();

	/**
	* Removes all edges from the pathway graph.
	*/
	void removeAllEdges();

	/**
	* Removes all vertices from the pathway graph.
	*/
	void removeAllVertices();

	/**
	* Removes the specified edge from the pathway graph.
	*/
	void removeEdge(Edge edge);

	/**
	* Removes the specified vertex from the pathway graph.
	*/
	void removeVertex(Vertex vertex);

    Object getObject();
    
    void setObject(Object object);
}
