package vp.graph;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.TexturePaint;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;

import vp.util.database.DBConstants;
import vp.visual.DecorationRenderer;
import vp.visual.LabeledRenderer;
import vp.visual.Renderer;
import vp.visual.paint.Painter;
import vp.visual.type.DefaultDecorationType1;
import vp.visual.type.Type;

/**
* A basic implementation of the <code>Decoration</code> interface.
* <code>PWEnzyme</code> is ultimately used to represent enzymes on the pathway graph, while <code>DecorationImp</code>
* is used for other types of decoration, e.g. Expression values or as a temporary form in preview windows and so forth.
*
* @see edu.yale.graph.Decoration
* @see edu.yale.graph.PWEnzyme
*/
public class PathwayDecoration
	extends AbstractDecoration
	implements LabeledElement, LabeledRenderer, TooltipElement, PathwayElement
{
	private Type type;

	private DataClass dataClass;

	public static final Type DEFAULT_TYPE = new DefaultDecorationType1();

	public static final int WEST = 1;
	public static final int EAST = -1;

	private final LabeledElementImp labeler;
	private Renderer renderer;
	private final TooltipElementImp tooltipper;
	private Type initialType;
	private int relPosition;

	public PathwayDecoration(PathwayDecoration decoration)
	{
		this.graph = decoration.getGraph();
		this.id = decoration.getID();
		obj = decoration.getDataClass();
		this.dataClass = decoration.getDataClass();

		this.relPosition = decoration.getRelPosition();

		String className = dataClass.getClassName();

		if ((className != null) && (className.equals("Enzyme")))
		{
			labeler = new LabeledElementImp((String)dataClass.get("ECNumber"));
		}
		else
		{
			labeler = new LabeledElementImp((String)dataClass.get("Name"));
		}

		tooltipper = new TooltipElementImp((String)dataClass.get("Name"));
		renderer =
			new DecorationRenderer(
				(DecorationRenderer)decoration.getRenderer(),
				this);
		this.type = renderer.getType();
		rescale();
	}

	public PathwayDecoration(
		Graph graph,
		int id,
		DataClass dataClass,
		DecorableElement parent,
		int relPosition)
	{
		this(graph, id, dataClass, null, parent, relPosition);
	}

	public PathwayDecoration(
		Graph graph,
		int id,
		DataClass dataClass,
		Type type,
		DecorableElement parent,
		int relPosition)
	{
		this.graph = graph;
		this.id = id;
		obj = dataClass;
		this.dataClass = dataClass;
		this.relPosition = relPosition;
		this.parent = parent;

		initialType =
			(dataClass.getStoredType() != null)
				? dataClass.getStoredType()
				: DEFAULT_TYPE;

		if (type == null)
			type = initialType;
		this.type = type;
		String className = dataClass.getClassName();

		if ((className != null) && (className.equals("Enzyme")))
		{
			labeler = new LabeledElementImp((String)dataClass.get("ECNumber"));
		}
		else
		{
			labeler = new LabeledElementImp((String)dataClass.get("Name"));
		}

		tooltipper = new TooltipElementImp((String)dataClass.get("Name"));
		renderer = new DecorationRenderer(this);
		rescale();
	}

	/**
	 * @return
	 */
	public Type getInitialType()
	{
		return initialType;
	}

	/**
	 * @return
	 */
	public LabeledElementImp getLabeler()
	{
		return labeler;
	}

	/**
	 * @return
	 */
	public int getRelPosition()
	{
		return relPosition;
	}

	/**
	 * @return
	 */
	public Renderer getRenderer()
	{
		return renderer;
	}

	/**
	 * @return
	 */
	public TooltipElementImp getTooltipper()
	{
		return tooltipper;
	}

	/**
	 * @param i
	 */
	public void setRelPosition(int i)
	{
		relPosition = i;
	}
	/* (non-Javadoc)
	 * @see vp.graph.LabeledElement#getLabel()
	 */
	public String getLabel()
	{
		return labeler.getLabel();
	}
	/* (non-Javadoc)
	 * @see vp.graph.LabeledElement#hasLabel()
	 */
	public boolean hasLabel()
	{
		return labeler.hasLabel();
	}
	/* (non-Javadoc)
	 * @see vp.graph.LabeledElement#setLabel(java.lang.String)
	 */
	public void setLabel(String label)
	{
		labeler.setLabel(label);
	}
	/* (non-Javadoc)
	 * @see vp.visual.LabeledRenderer#getFillColor()
	 */
	public Color getFillColor()
	{
		return ((DecorationRenderer)renderer).getFillColor();
	}
	/* (non-Javadoc)
	 * @see vp.visual.LabeledRenderer#getFont()
	 */
	public Font getFont()
	{
		return ((DecorationRenderer)renderer).getFont();
	}
	/* (non-Javadoc)
	 * @see vp.visual.LabeledRenderer#getFontColor()
	 */
	public Color getFontColor()
	{
		return ((DecorationRenderer)renderer).getFontColor();
	}
	/* (non-Javadoc)
	 * @see vp.visual.LabeledRenderer#getFontMetrics()
	 */
	public FontMetrics getFontMetrics()
	{
		return ((DecorationRenderer)renderer).getFontMetrics();
	}
	/* (non-Javadoc)
	 * @see vp.visual.LabeledRenderer#rescale()
	 */
	public void rescale()
	{
		((DecorationRenderer)renderer).rescale();
	}
	/* (non-Javadoc)
	 * @see vp.visual.LabeledRenderer#setFillColor(java.awt.Color)
	 */
	public void setFillColor(Color fillColor)
	{
		((DecorationRenderer)renderer).setFillColor(fillColor);
	}
	/* (non-Javadoc)
	 * @see vp.visual.LabeledRenderer#setFont(java.awt.Font)
	 */
	public void setFont(Font font)
	{
		((DecorationRenderer)renderer).setFont(font);
	}
	/* (non-Javadoc)
	 * @see vp.visual.LabeledRenderer#setFontColor(java.awt.Color)
	 */
	public void setFontColor(Color fontColor)
	{
		((DecorationRenderer)renderer).setFontColor(fontColor);
	}
	/* (non-Javadoc)
	 * @see vp.visual.LabeledRenderer#setLocation(double, double)
	 */
	public void setLocation(double x, double y)
	{
		((DecorationRenderer)renderer).setLocation(x, y);
	}
	/* (non-Javadoc)
	 * @see vp.visual.Renderer#containsPoint(int, int)
	 */
	public boolean containsPoint(int x, int y)
	{
		return ((DecorationRenderer)renderer).containsPoint(x, y);
	}
	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getBounds()
	 */
	public Rectangle getBounds()
	{
		return ((DecorationRenderer)renderer).getBounds();
	}
	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getBounds2D()
	 */
	public Rectangle2D getBounds2D()
	{
		return ((DecorationRenderer)renderer).getBounds2D();
	}
	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getGeneralPath()
	 */
	public GeneralPath getGeneralPath()
	{
		return ((DecorationRenderer)renderer).getGeneralPath();
	}
	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getOutlineColor()
	 */
	public Color getOutlineColor()
	{
		return ((DecorationRenderer)renderer).getOutlineColor();
	}
	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getPainter()
	 */
	public Painter getPainter()
	{
		return ((DecorationRenderer)renderer).getPainter();
	}
	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getTexturePaint()
	 */
	public TexturePaint getTexturePaint()
	{
		return ((DecorationRenderer)renderer).getTexturePaint();
	}
	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getType()
	 */
	public Type getType()
	{
		return type;
	}
	/* (non-Javadoc)
	 * @see vp.visual.Renderer#paint(java.awt.Graphics2D)
	 */
	public void paint(Graphics2D g2)
	{
		((DecorationRenderer)renderer).paint(g2);
	}
	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setGeneralPath(java.awt.geom.GeneralPath)
	 */
	public void setGeneralPath(GeneralPath path)
	{
		((DecorationRenderer)renderer).setGeneralPath(path);
	}
	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setOutlineColor(java.awt.Color)
	 */
	public void setOutlineColor(Color outlineColor)
	{
		((DecorationRenderer)renderer).setOutlineColor(outlineColor);
	}
	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setPainter(vp.visual.paint.Painter)
	 */
	public void setPainter(Painter painter)
	{
		((DecorationRenderer)renderer).setPainter(painter);
	}
	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setTexturePaint(java.awt.TexturePaint)
	 */
	public void setTexturePaint(TexturePaint tp)
	{
		((DecorationRenderer)renderer).setTexturePaint(tp);
	}
	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setType(vp.visual.type.Type)
	 */
	public void setType(Type type)
	{
		this.type = type;
		((DecorationRenderer)renderer).setType(type);
	}
	/* (non-Javadoc)
	 * @see vp.graph.TooltipElement#getToolTip()
	 */
	public String getToolTip()
	{
		return tooltipper.getToolTip();
	}
	/* (non-Javadoc)
	 * @see vp.graph.TooltipElement#hasToolTip()
	 */
	public boolean hasToolTip()
	{
		return tooltipper.hasToolTip();
	}
	/* (non-Javadoc)
	 * @see vp.graph.TooltipElement#getTipEnabled()
	 */
	public boolean isTipEnabled()
	{
		return tooltipper.isTipEnabled();
	}
	/* (non-Javadoc)
	 * @see vp.graph.TooltipElement#setTipEnabled(boolean)
	 */
	public void setTipEnabled(boolean enabled)
	{
		tooltipper.setTipEnabled(enabled);
	}
	/* (non-Javadoc)
	 * @see vp.graph.TooltipElement#setToolTip(java.lang.String)
	 */
	public void setToolTip(String tooltip)
	{
		tooltipper.setToolTip(tooltip);
	}

	/**
	 * @return
	 */
	public String getShape()
	{
		return ((DecorationRenderer)renderer).getShape();
	}

	public void setShape(String shape)
	{
		((DecorationRenderer)renderer).setShape(shape);
	}
	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getDataClass()
	 */
	public DataClass getDataClass()
	{
		return dataClass;
	}
	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setDataClass(vp.graph.DataClass)
	 */
	public void setDataClass(DataClass dataClass)
	{
		this.dataClass = dataClass;
	}
	/* (non-Javadoc)
	 * @see vp.graph.PathwayElement#setRenderer(vp.visual.Renderer)
	 */
	public void setRenderer(Renderer renderer)
	{
		this.renderer = renderer;
	}
	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getElement()
	 */
	public PathwayElement getElement()
	{
		return renderer.getElement();
	}
	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getElement(vp.graph.PathwayElement)
	 */
	public void setElement(PathwayElement element)
	{
		renderer.setElement(element);
	}
	/* (non-Javadoc)
	 * @see vp.graph.PathwayElement#getElementType()
	 */
	public int getElementType()
	{
		return DBConstants.DECORATION;
	}

}
