package vp.graph;

import vp.util.database.DBConstants;
import vp.visual.EdgeRenderer;
import vp.visual.Renderer;
import vp.visual.paint.Painter;
import vp.visual.type.DefaultEdgeType1;
import vp.visual.type.Type;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.TexturePaint;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D.Double;
import java.util.ArrayList;
import java.util.ListIterator;

/**
* A basic implementation of the <code>Edge</code> interface.
* While, <code>Link</code> is ultimately used to represent links on the pathway graph, <code>EdgeImp</code>
* is useful as a temporary form in preview windows and so forth.
*
* @see edu.yale.graph.Edge
* @see edu.yale.graph.Link
*/
public class PathwayEdge
	extends AbstractEdge
	implements DecorableElement, Renderer, PathwayElement
{
	private Type type;

	private DataClass dataClass;

	public static final Type DEFAULT_TYPE = new DefaultEdgeType1();

	public static final String BOTH = "Both Ways";
	public static final String FORWARD = "Forward";
	public static final String BACKWARD = "Backward";
	public static final String NEITHER = "Neither";

	private Type initialType;
	private final DecorableElementImp decorator;
	private Renderer renderer;

	public PathwayEdge(PathwayEdge edge)
	{
		this.graph = edge.getGraph();
		this.id = edge.getID();
		obj = edge.getDataClass();
		this.dataClass = edge.getDataClass();

		this.vertexA = edge.getVertexA();
		this.vertexB = edge.getVertexB();
		this.direction = edge.getDirection();

		initialType = edge.getInitialType();
		decorator = new DecorableElementImp();

		renderer = new EdgeRenderer((EdgeRenderer)edge.getRenderer(), this);
        this.type = renderer.getType();
        
        ListIterator decs = edge.getDecorationIterator();
        while(decs.hasNext())
        {
            addDecoration(new PathwayDecoration((PathwayDecoration)decs.next()));
        }

	}

	public PathwayEdge(
		Graph graph,
		int id,
		DataClass dataClass,
		Vertex vertexA,
		Vertex vertexB,
		String direction)
	{
		this(graph, id, dataClass, null, vertexA, vertexB, direction);
	}

	public PathwayEdge(
		Graph graph,
		int id,
		DataClass dataClass,
		Type type,
		Vertex vertexA,
		Vertex vertexB,
		String direction)
	{
		this.graph = graph;
		this.id = id;
		this.vertexA = vertexA;
		this.vertexB = vertexB;
		this.direction = direction;
		obj = dataClass;
		this.dataClass = dataClass;
		

		initialType =
			(dataClass.getStoredType() != null)
				? dataClass.getStoredType()
				: DEFAULT_TYPE;

		if (type == null)
			type = initialType;
        this.type = type;
		decorator = new DecorableElementImp();
		renderer = new EdgeRenderer(this);
	}

	public Type getInitialType()
	{
		return initialType;
	}

	public DecorableElementImp getDecorator()
	{
		return decorator;
	}

	public Renderer getRenderer()
	{
		return renderer;
	}

	/* (non-Javadoc)
	 * @see vp.graph.DecorableElement#addDecoration(vp.graph.Decoration)
	 */
	public void addDecoration(Decoration dec)
	{
		decorator.addDecoration(dec);
	}

	/* (non-Javadoc)
	 * @see vp.graph.DecorableElement#addDecListener(vp.graph.DecorationListener)
	 */
	public void addDecListener(DecorationListener listener)
	{
		decorator.addDecListener(listener);
	}

	/* (non-Javadoc)
	 * @see vp.graph.DecorableElement#containsDecoration(vp.graph.Decoration)
	 */
	public boolean containsDecoration(Decoration dec)
	{
		return decorator.containsDecoration(dec);
	}

	/* (non-Javadoc)
	 * @see vp.graph.DecorableElement#getDecorationCount()
	 */
	public int getDecorationCount()
	{
		return decorator.getDecorationCount();
	}

	/* (non-Javadoc)
	 * @see vp.graph.DecorableElement#getDecorationIterator()
	 */
	public ListIterator getDecorationIterator()
	{
		return decorator.getDecorationIterator();
	}

	/* (non-Javadoc)
	 * @see vp.graph.DecorableElement#getDecList()
	 */
	public ArrayList getDecList()
	{
		return decorator.getDecList();
	}

	/* (non-Javadoc)
	 * @see vp.graph.DecorableElement#removeAllDecorations()
	 */
	public void removeAllDecorations()
	{
		decorator.removeAllDecorations();
	}

	/* (non-Javadoc)
	 * @see vp.graph.DecorableElement#removeDecoration(vp.graph.Decoration)
	 */
	public void removeDecoration(Decoration dec)
	{
		decorator.removeDecoration(dec);
	}

	/* (non-Javadoc)
	 * @see vp.graph.DecorableElement#removeDecListener(vp.graph.DecorationListener)
	 */
	public void removeDecListener(DecorationListener listener)
	{
		decorator.removeDecListener(listener);
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#containsPoint(int, int)
	 */
	public boolean containsPoint(int x, int y)
	{
		return renderer.containsPoint(x, y);
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getBounds()
	 */
	public Rectangle getBounds()
	{
		return renderer.getBounds();
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getBounds2D()
	 */
	public Rectangle2D getBounds2D()
	{
		return renderer.getBounds2D();
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getGeneralPath()
	 */
	public GeneralPath getGeneralPath()
	{
		return renderer.getGeneralPath();
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getOutlineColor()
	 */
	public Color getOutlineColor()
	{
		return renderer.getOutlineColor();
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getPainter()
	 */
	public Painter getPainter()
	{
		return renderer.getPainter();
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getTexturePaint()
	 */
	public TexturePaint getTexturePaint()
	{
		return renderer.getTexturePaint();
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getType()
	 */
	public Type getType()
	{
		return type;
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#paint(java.awt.Graphics2D)
	 */
	public void paint(Graphics2D g2)
	{
		((EdgeRenderer)renderer).paint(g2);
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setGeneralPath(java.awt.geom.GeneralPath)
	 */
	public void setGeneralPath(GeneralPath path)
	{
		renderer.setGeneralPath(path);
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setOutlineColor(java.awt.Color)
	 */
	public void setOutlineColor(Color outlineColor)
	{
		renderer.setOutlineColor(outlineColor);
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setPainter(vp.visual.paint.Painter)
	 */
	public void setPainter(Painter painter)
	{
		renderer.setPainter(painter);
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setTexturePaint(java.awt.TexturePaint)
	 */
	public void setTexturePaint(TexturePaint tp)
	{
		renderer.setTexturePaint(tp);
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setType(vp.visual.type.Type)
	 */
	public void setType(Type type)
	{
		this.type = type;
		renderer.setType(type);
	}

	/**
	 * @return
	 */
	public GeneralPath getClickPath()
	{
		return ((EdgeRenderer)renderer).getClickPath();
	}

	/**
	 * @return
	 */
	public float getCurvature()
	{
		return ((EdgeRenderer)renderer).getCurvature();
	}

	/**
	 * @param edge
	 * @param linkRatio
	 * @return
	 */
	public Double getLinkPosition(double linkRatio)
	{
		return ((EdgeRenderer)renderer).getLinkPosition(this, linkRatio);
	}

	/**
	 * @param path
	 */
	public void setClickPath(GeneralPath path)
	{
		((EdgeRenderer)renderer).setClickPath(path);
	}

	/**
	 * @param f
	 */
	public void setCurvature(float f)
	{
		((EdgeRenderer)renderer).setCurvature(f);
	}

	/* (non-Javadoc)
	 * @see vp.graph.DecorableElement#getDecoration(int)
	 */
	public Decoration getDecoration(int index)
	{
		return decorator.getDecoration(index);
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getDataClass()
	 */
	public DataClass getDataClass()
	{
		return dataClass;
	}

	/**
	 * @return
	 */
	public String getShape()
	{
		return ((EdgeRenderer)renderer).getShape();
	}

	public void setShape(String shape)
	{
		((EdgeRenderer)renderer).setShape(shape);
	}

	/**
	 * 
	 */
	//	public void setTexturePaint()
	//	{
	//		((EdgeRenderer)renderer).setTexturePaint();
	//	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setDataClass(vp.graph.DataClass)
	 */
	public void setDataClass(DataClass dataClass)
	{
		this.dataClass = dataClass;
	}

	/* (non-Javadoc)
	 * @see vp.graph.PathwayElement#setRenderer(vp.visual.Renderer)
	 */
	public void setRenderer(Renderer renderer)
	{
		this.renderer = renderer;
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getElement()
	 */
	public PathwayElement getElement()
	{
		return renderer.getElement();
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setElement(vp.graph.PathwayElement)
	 */
	public void setElement(PathwayElement element)
	{
		renderer.setElement(element);
	}

	/* (non-Javadoc)
	 * @see vp.graph.PathwayElement#getElementType()
	 */
	public int getElementType()
	{
		return DBConstants.EDGE;
	}

}
