package vp.graph;

import vp.gui.dialogs.PathwayDisplayPanel;
import vp.main.MainFrame;
import vp.util.EditEdgeList;
import vp.util.EditVertexList;
import vp.util.SaveType;
import vp.util.database.DBConstants;
import vp.util.database.DBQuery;
import vp.visual.GraphRenderer;
import vp.visual.VPGraphRenderer;
import vp.visual.graphics.AnimatedElement;
import vp.visual.paint.DecorationPainterFactory;
import vp.visual.paint.EdgePainterFactory;
import vp.visual.paint.VertexPainterFactory;
import vp.visual.type.Type;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;

/**
 * The standard implementation of <code>Graph</code>.
 * 
 * @see edu.yale.graph.Graph
 */
public class PathwayGraph extends AbstractGraph implements
    GraphRenderer
{
    private boolean isExperimentData;

    private boolean isGeneData = false;

    private Experiment experiment;

    private GraphDataClass graphData;

    private MainFrame parent;

    private Map localTypes;

    private EditVertexList highlightedVertices;

    private EditEdgeList highlightedEdges;

    private List newDataClasses;

    private List availableVertexTypes;

    private List availableEdgeTypes;

    private List availableDecorationTypes;

    private List vertexClasses, edgeClasses, decorationClasses;

    private Map tableNames, defaultTypes, classNames;

    private VPGraphRenderer renderer;

    /**
     * Creates a new pathway graph with all new data.
     */
    public PathwayGraph(MainFrame parent, int id, GraphDataClass gdcl)
    {
        this.parent = parent;
        this.id = id;
        obj = gdcl;
        this.graphData = gdcl;

        if (gdcl != null)
            localTypes = DBQuery.getLocalTypes(gdcl.getObjectID());

        vertexList = new ArrayList();
        edgeList = new ArrayList();
        listeners = new ArrayList();
        newDataClasses = new ArrayList();
        highlightedEdges = new EditEdgeList(this);
        highlightedVertices = new EditVertexList(this);

        vertexClasses = initClasses(DBConstants.VERTEX);
        edgeClasses = initClasses(DBConstants.EDGE);
        decorationClasses = initClasses(DBConstants.DECORATION);

        tableNames = initTableNames();
        defaultTypes = initDefaultTypes();
        classNames = initClassNames();

        availableVertexTypes = initAvailableVertexTypes();
        availableEdgeTypes = initAvailableEdgeTypes();
        availableDecorationTypes = initAvailableDecorationTypes();

        renderer = new VPGraphRenderer(this);
        VisualGraphListener vgListener = new VisualGraphListener(this);
        addGraphListener(vgListener);
    }

    private List initClasses(int elementType)
    {
        // Get local classes
        List classes = DBQuery.getClasses((String) graphData
            .get("id"), elementType);

        // compare with the defaults
        List globalClasses = parent.getClasses(elementType);
        for (int i = 0; i < globalClasses.size(); i++)
        {
            Integer classID = (Integer) globalClasses.get(i);

            // Add a global class if not overriden for the specific
            // pathway
            if (!classes.contains(classID))
                classes.add(classID);
        }
        return classes;
    }

    private Map initTableNames()
    {
        Map tableNames = new HashMap();

        for (int i = 0; i < vertexClasses.size(); i++)
        {
            Integer classID = (Integer) vertexClasses.get(i);
            String name = DBQuery.getTableName((String) graphData
                .get("id"), classID.intValue());
            if (name == null)
                name = parent.getTableName(classID.intValue());
            tableNames.put(classID, name);
        }

        for (int i = 0; i < edgeClasses.size(); i++)
        {
            Integer classID = (Integer) edgeClasses.get(i);
            String name = DBQuery.getTableName((String) graphData
                .get("id"), classID.intValue());
            if (name == null)
                name = parent.getTableName(classID.intValue());
            tableNames.put(classID, name);
        }

        for (int i = 0; i < decorationClasses.size(); i++)
        {
            Integer classID = (Integer) decorationClasses.get(i);
            String name = DBQuery.getTableName((String) graphData
                .get("id"), classID.intValue());
            if (name == null)
                name = parent.getTableName(classID.intValue());
            tableNames.put(classID, name);
        }

        return tableNames;
    }

    private Map initClassNames()
    {
        Map classNames = new HashMap();

        for (int i = 0; i < vertexClasses.size(); i++)
        {
            Integer classID = (Integer) vertexClasses.get(i);
            String name = DBQuery.getClassName((String) graphData
                .get("id"), classID.intValue());
            if (name == null)
                name = parent.getClassName(classID.intValue());
            tableNames.put(classID, name);
        }

        for (int i = 0; i < edgeClasses.size(); i++)
        {
            Integer classID = (Integer) edgeClasses.get(i);
            String name = DBQuery.getClassName((String) graphData
                .get("id"), classID.intValue());
            if (name == null)
                name = parent.getClassName(classID.intValue());
            tableNames.put(classID, name);
        }

        for (int i = 0; i < decorationClasses.size(); i++)
        {
            Integer classID = (Integer) decorationClasses.get(i);
            String name = DBQuery.getClassName((String) graphData
                .get("id"), classID.intValue());
            if (name == null)
                name = parent.getClassName(classID.intValue());
            tableNames.put(classID, name);
        }

        return tableNames;
    }

    private Map initDefaultTypes()
    {
        Map defaultTypes = new HashMap();

        for (int i = 0; i < vertexClasses.size(); i++)
        {
            Integer classID = (Integer) vertexClasses.get(i);
            Type type = DBQuery.getDefaultType((String) graphData
                .get("id"), classID.intValue());
            if (type == null)
                type = parent.getDefaultType(classID.intValue());
            defaultTypes.put(classID, type);
        }

        for (int i = 0; i < edgeClasses.size(); i++)
        {
            Integer classID = (Integer) edgeClasses.get(i);
            Type type = DBQuery.getDefaultType((String) graphData
                .get("id"), classID.intValue());
            if (type == null)
                type = parent.getDefaultType(classID.intValue());
            defaultTypes.put(classID, type);
        }

        for (int i = 0; i < decorationClasses.size(); i++)
        {
            Integer classID = (Integer) decorationClasses.get(i);
            Type type = DBQuery.getDefaultType((String) graphData
                .get("id"), classID.intValue());
            if (type == null)
                type = parent.getDefaultType(classID.intValue());
            defaultTypes.put(classID, type);
        }

        return defaultTypes;
    }

    /**
     * @return
     */
    private List initAvailableVertexTypes()
    {
        List types = new ArrayList();
        // Get all of the local defaults
        for (int i = 0; i < vertexClasses.size(); i++)
            types.add((Type) defaultTypes.get(vertexClasses.get(i)));
        // Add any types saved in flat files
        types.addAll(SaveType.getVertexTypes());
        return types;
    }

    /**
     * @return
     */
    private List initAvailableEdgeTypes()
    {
        List types = new ArrayList();
        // Get all of the local defaults
        for (int i = 0; i < edgeClasses.size(); i++)
            types.add((Type) defaultTypes.get(edgeClasses.get(i)));
        // Add any types saved in flat files
        types.addAll(SaveType.getEdgeTypes());
        return types;
    }

    /**
     * @return
     */
    private List initAvailableDecorationTypes()
    {
        List types = new ArrayList();
        // Get all of the local defaults
        for (int i = 0; i < decorationClasses.size(); i++)
            types.add((Type) defaultTypes.get(decorationClasses
                .get(i)));
        // Add any types saved in flat files
        types.addAll(SaveType.getDecorationTypes());
        return types;
    }

    /**
     * Adds the user-defined compound to the list of new compounds.
     */
    public void addNewDataClass(DataClass dcl)
    {
        newDataClasses.add(dcl);
    }

    /**
     * Returns an <code>ArrayList</code> containing all available
     * visual decoration types. This will be all default types, all
     * saved custom user types, all temporary user-defined types and
     * any other unsaved types included in the current pathway graph.
     */
    public List getAvailableDecorationTypes()
    {
        return availableDecorationTypes;
    }

    public void updateAvailableDecorationTypes()
    {
        availableDecorationTypes = initAvailableDecorationTypes();
    }

    /**
     * Returns an <code>ArrayList</code> containing all available
     * visual edge types. This will be all default types, all saved
     * custom user types, all temporary user-defined types and any
     * other unsaved types included in the current pathway graph.
     */
    public List getAvailableEdgeTypes()
    {
        return availableEdgeTypes;
    }

    public void updateAvailableEdgeTypes()
    {
        availableEdgeTypes = initAvailableEdgeTypes();
    }

    /**
     * Returns an <code>ArrayList</code> containing all available
     * visual vertex types. This will be all default types, all saved
     * custom user types, all temporary user-defined types and any
     * other unsaved types included in the current pathway graph.
     */
    public List getAvailableVertexTypes()
    {
        return availableVertexTypes;
    }

    public void updateAvailableVertexTypes()
    {
        availableVertexTypes = initAvailableVertexTypes();
    }

    /**
     * Removes the specified user-defined compound from the list of
     * new compounds.
     */
    public void removeNewDataClass(DataClass dcl)
    {
        newDataClasses.remove(dcl);
    }

    /**
     * @return
     */
    public List getNewDataClasses()
    {
        return newDataClasses;
    }

    /**
     * @return
     */
    public Type getLocalType(int classID)
    {
        return (Type) localTypes.get(new Integer(classID));
    }

    public void setLocalType(int classID, Type type)
    {
        localTypes.put(new Integer(classID), type);
    }

    /**
     * @return
     */
    public GraphDataClass getGraphDataClass()
    {
        return graphData;
    }

    /**
     * @param pathway
     */
    public void setGraphDataClass(GraphDataClass graphData)
    {
        this.graphData = graphData;
        if (graphData != null)
            localTypes = DBQuery.getLocalTypes(graphData
                .getObjectID());
    }

    /**
     * Returns the full list of edges that have been highlighted for
     * modification by the user.
     */
    public EditEdgeList getHighlightedEdges()
    {
        return highlightedEdges;
    }

    /**
     * Returns the full list of vertices that have been highlighted
     * for modification by the user.
     */
    public EditVertexList getHighlightedVertices()
    {
        return highlightedVertices;
    }

    public void clearHighlightedElements()
    {
        highlightedEdges.clear();
        highlightedVertices.clear();
        renderer.clearHighlightedElements();
    }

    /**
     * @return
     */
    public MainFrame getParent()
    {
        return parent;
    }

    /*
     * (non-Javadoc)
     * 
     * @see vp.visual.GraphRenderer#addContainer(java.awt.Container)
     */
    public void addContainer(Container c)
    {
        renderer.addContainer(c);
    }

    /*
     * (non-Javadoc)
     * 
     * @see vp.visual.GraphRenderer#getContainers()
     */
    public List getContainers()
    {
        return renderer.getContainers();
    }

    /*
     * (non-Javadoc)
     * 
     * @see vp.visual.GraphRenderer#getSelectedDecoration(int, int)
     */
    public PathwayDecoration getSelectedDecoration(int x, int y)
    {
        return renderer.getSelectedDecoration(x, y);
    }

    /*
     * (non-Javadoc)
     * 
     * @see vp.visual.GraphRenderer#getSelectedEdge(int, int)
     */
    public PathwayEdge getSelectedEdge(int x, int y)
    {
        return renderer.getSelectedEdge(x, y);
    }

    /*
     * (non-Javadoc)
     * 
     * @see vp.visual.GraphRenderer#getMaxSize()
     */
    public Dimension getMaxSize()
    {
        return renderer.getMaxSize();
    }

    /*
     * (non-Javadoc)
     * 
     * @see vp.visual.GraphRenderer#getSelectedVertex(int, int)
     */
    public PathwayVertex getSelectedVertex(int x, int y)
    {
        return renderer.getSelectedVertex(x, y);
    }

    /*
     * (non-Javadoc)
     * 
     * @see vp.visual.GraphRenderer#getDecorationPainterFactory()
     */
    public DecorationPainterFactory getDecorationPainterFactory()
    {
        return renderer.getDecorationPainterFactory();
    }

    /*
     * (non-Javadoc)
     * 
     * @see vp.visual.GraphRenderer#getEdgePainterFactory()
     */
    public EdgePainterFactory getEdgePainterFactory()
    {
        return renderer.getEdgePainterFactory();
    }

    /*
     * (non-Javadoc)
     * 
     * @see vp.visual.GraphRenderer#getVertexPainterFactory()
     */
    public VertexPainterFactory getVertexPainterFactory()
    {
        return renderer.getVertexPainterFactory();
    }

    /*
     * (non-Javadoc)
     * 
     * @see vp.visual.GraphRenderer#isLocked()
     */
    public boolean isLocked()
    {
        return renderer.isLocked();
    }

    /*
     * (non-Javadoc)
     * 
     * @see vp.visual.GraphRenderer#paint()
     */
    public void paint(Graphics2D g2)
    {
        renderer.paint(g2);
    }

    /*
     * (non-Javadoc)
     * 
     * @see vp.visual.GraphRenderer#removeContainter(java.awt.Container)
     */
    public void removeContainer(Container c)
    {
        renderer.removeContainer(c);
    }

    /*
     * (non-Javadoc)
     * 
     * @see vp.visual.GraphRenderer#repaint()
     */
    public void repaint()
    {
        renderer.repaint();
    }

    /*
     * (non-Javadoc)
     * 
     * @see vp.visual.GraphRenderer#setExperimentData(java.util.Map)
     */
    public void setExperimentData(Map data)
    {
        renderer.setExperimentData(data);
    }

    /*
     * (non-Javadoc)
     * 
     * @see vp.visual.GraphRenderer#setLocked(boolean)
     */
    public void setLocked(boolean locked)
    {
        renderer.setLocked(locked);
    }

    /*
     * (non-Javadoc)
     * 
     * @see vp.visual.GraphRenderer#setDecorationPainterFactory(vp.visual.paint.DecorationPainterFactory)
     */
    public void setDecorationPainterFactory(
        DecorationPainterFactory dpf)
    {
        renderer.setDecorationPainterFactory(dpf);
    }

    /*
     * (non-Javadoc)
     * 
     * @see vp.visual.GraphRenderer#setEdgePainterFactory(vp.visual.paint.EdgePainterFactory)
     */
    public void setEdgePainterFactory(EdgePainterFactory epf)
    {
        renderer.setEdgePainterFactory(epf);
    }

    /*
     * (non-Javadoc)
     * 
     * @see vp.visual.GraphRenderer#setVertexPainterFactory(vp.visual.paint.VertexPainterFactory)
     */
    public void setVertexPainterFactory(VertexPainterFactory vpf)
    {
        renderer.setVertexPainterFactory(vpf);
    }

    /**
     * 
     */
    public void removeAllAnimatedElements()
    {
        renderer.removeAllAnimatedElements();
    }

    /**
     * 
     */
    public Experiment getExperiment()
    {
        return experiment;
    }

    /**
     * @param exp
     * @param gradient
     * @param selectedExperiment1
     */
    public void setExpressionDataByGene(Map exp, VPGradient gradient,
        Experiment experiment)
    {
        DecimalFormat nf = (DecimalFormat) DecimalFormat
            .getInstance();
        nf.setMaximumFractionDigits(2);
        nf.setGroupingUsed(false);

        List edgesToAdd = new ArrayList();
        List edgesToDelete = new ArrayList();

        for (int j = 0; j < edgeList.size(); j++)
        {

            PathwayEdge oldEdge = (PathwayEdge) edgeList.get(j);
            PathwayEdge newEdge = new PathwayEdge(oldEdge.getGraph(),
                DBQuery.getUniqueID(), oldEdge.getDataClass(),
                oldEdge.getType(), oldEdge.getVertexA(), oldEdge
                    .getVertexB(), oldEdge.getDirection());
            ArrayList newDecs = new ArrayList();
            ArrayList enzymes = new ArrayList();
            ListIterator decIter = oldEdge.getDecorationIterator();
            while (decIter.hasNext())
            {
                PathwayDecoration currDec = (PathwayDecoration) decIter
                    .next();
                String className = currDec.getDataClass()
                    .getClassName();
                if ((className != null)
                    && (className.equals("Enzyme")))
                {
                    DataClass enzClass = currDec.getDataClass();
                    enzymes.add(enzClass);

                    if (exp.containsKey(enzClass.get("ECNumber")))
                    {
                        HashMap expValueMap = (HashMap) exp
                            .get(enzClass.get("ECNumber"));
                        Iterator valueSetIter = expValueMap.keySet()
                            .iterator();
                        while (valueSetIter.hasNext())
                        {
                            DataClass gene = (DataClass) valueSetIter
                                .next();

                            String geneValue = expValueMap.get(gene)
                                .toString();
                            try
                            {
                                geneValue = nf.format(
                                    Double.parseDouble(geneValue))
                                    .toString();
                            }
                            catch (NumberFormatException nfe)
                            {
                                geneValue = "N/A";
                            }
                            DataClass valueClass = new DataClass(
                                "expValue", this);
                            valueClass.setClassName("Expression");
                            valueClass.put("Name", geneValue);
                            PathwayDecoration pwgene = new PathwayDecoration(
                                this, DBQuery.getUniqueID(), gene,
                                newEdge, 1);
                            PathwayDecoration gValue = new PathwayDecoration(
                                this, DBQuery.getUniqueID(),
                                valueClass, gradient
                                    .getType(geneValue), newEdge, -1);
                            newDecs.add(pwgene);
                            newDecs.add(gValue);

                        }

                    }
                }

                else if ((className != null)
                    && (className.equals("Gene")))
                {
                    DataClass geneClass = currDec.getDataClass();

                    if (exp.containsKey(geneClass.get("Name")))
                    {
                        String geneValue = exp.get(
                            geneClass.get("Name")).toString();
                        try
                        {
                            geneValue = nf.format(
                                Double.parseDouble(geneValue))
                                .toString();
                        }
                        catch (NumberFormatException nfe)
                        {
                            geneValue = "N/A";
                        }
                        DataClass valueClass = new DataClass(
                            "expValue", this);
                        valueClass.setClassName("Expression");
                        valueClass.put("Name", geneValue);
                        PathwayDecoration pwgene = new PathwayDecoration(
                            this, DBQuery.getUniqueID(), geneClass,
                            newEdge, 1);
                        PathwayDecoration gValue = new PathwayDecoration(
                            this, DBQuery.getUniqueID(), valueClass,
                            gradient.getType(geneValue), newEdge, -1);
                        newDecs.add(pwgene);
                        newDecs.add(gValue);

                    }
                }
            }

            for (int i = 0; i < newDecs.size(); i++)
            {
                if (newEdge != null)
                {
                    try
                    {
                        newEdge
                            .addDecoration((PathwayDecoration) newDecs
                                .get(i));
                    }
                    catch (Exception e)
                    {
                        e.printStackTrace();
                    }
                }
            }
            edgesToDelete.add(oldEdge);
            edgesToAdd.add(newEdge);
        }
        for (int i = 0; i < edgesToDelete.size(); i++)
        {
            try
            {
                removeEdge((PathwayEdge) edgesToDelete.get(i));

            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
        for (int i = 0; i < edgesToAdd.size(); i++)
        {
            try
            {
                addEdge((PathwayEdge) edgesToAdd.get(i));
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
        repaint();
        isGeneData = true;
        setExperimentData(true);
        setExperiment(experiment);
        parent.addToDisplayPanel(new PathwayDisplayPanel(parent));
    }

    /**
     * @param exp
     */
    private void setExperiment(Experiment experiment)
    {
        this.experiment = experiment;
    }

    /**
     * @return
     */
    public boolean isExperimentData()
    {
        return isExperimentData;
    }

    public boolean isGeneData()
    {
        return isGeneData;
    }

    /**
     * @param ae
     */
    public void addAnimatedElement(AnimatedElement ae)
    {
        renderer.addAnimatedElement(ae);
    }

    /**
     * @param b
     */
    public void setExperimentData(boolean b)
    {
        isExperimentData = b;
    }

    /**
     * @param i
     * @param localCompoundType
     */
    public void setDefaultType(int i, Type localCompoundType)
    {
        renderer.setDefaultType(i, localCompoundType);
    }

    /**
     * @param i
     * @return
     */
    public Type getDefaultType(int i)
    {
        return null;
    }

    /**
     * @return
     */
    public String getAnnotation()
    {
        return null;
    }

    /**
     * @param annotation
     */
    public void setAnnotation(String annotation)
    {
    }

    /**
     * 
     */
    public void clearAll()
    {
        highlightedEdges.clear();
        highlightedVertices.clear();

        List allVerts = new ArrayList(vertexList);

        for (int i = 0; i < allVerts.size(); i++)
            removeVertex((PathwayVertex) allVerts.get(i));

        // just in case
        List allEdges = new ArrayList(edgeList);
        for (int i = 0; i < allEdges.size(); i++)
            removeEdge((PathwayEdge) allEdges.get(i));
    }

    public List getNeighborList(Vertex vertex)
    {
        List neighbors = new ArrayList();
        ListIterator iter = getEdgeIterator();
        do
        {
            if (!iter.hasNext())
                break;
            Edge e = (Edge) iter.next();
            if (e.getVertexA() == vertex || e.getVertexB() == vertex)
                neighbors.add(e.getOppositeVertex(vertex));
        }
        while (true);
        return neighbors;
    }

    public void swapEnzymesForGenes(boolean repaint)
    {
        List edgesToAdd = new ArrayList();
        List edgesToDelete = new ArrayList();

        for (int i = 0; i < edgeList.size(); i++)
        {
            PathwayEdge oldEdge = (PathwayEdge) edgeList.get(i);
            PathwayEdge newEdge = new PathwayEdge(oldEdge.getGraph(),
                DBQuery.getUniqueID(), oldEdge.getDataClass(),
                oldEdge.getType(), oldEdge.getVertexA(), oldEdge
                    .getVertexB(), oldEdge.getDirection());
            List newDecs = new ArrayList();

            List genes = (List) newEdge.getDataClass().get("Genes");
            ListIterator giter = genes.listIterator();

            while (giter.hasNext())
            {
                DataClass current = (DataClass) giter.next();
                PathwayDecoration pwe = new PathwayDecoration(this,
                    DBQuery.getUniqueID(), current, newEdge, 1);

                newEdge.addDecoration(pwe);
            }

            edgesToDelete.add(oldEdge);
            edgesToAdd.add(newEdge);
        }

        for (int i = 0; i < edgesToDelete.size(); i++)
        {
            removeEdge((PathwayEdge) edgesToDelete.get(i));
        }

        for (int i = 0; i < edgesToAdd.size(); i++)
        {
            addEdge((PathwayEdge) edgesToAdd.get(i));
        }

        if (repaint)
            repaint();
        isGeneData = true;
    }
}
