package vp.graph;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.TexturePaint;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;

import vp.util.database.DBConstants;
import vp.visual.LabeledRenderer;
import vp.visual.Renderer;
import vp.visual.VertexRenderer;
import vp.visual.paint.Painter;
import vp.visual.type.DefaultVertexType1;
import vp.visual.type.Type;

/**
* A basic implementation of the <code>Vertex</code> interface.
* While, <code>PWVertex</code> is ultimately used to represent compounds on the pathway graph, <code>VertexImp</code>
* is useful as a temporary form in preview windows and so forth.
*
* @see edu.yale.graph.Vertex
* @see edu.yale.graph.PWVertex
*/

public class PathwayVertex
	extends AbstractVertex
	implements LabeledRenderer, LabeledElement, PathwayElement
{
	private DataClass dataClass;
	private double onLinkPosition;
	private int onLink;
	private final LabeledElementImp labeler;
	private Renderer renderer;
	private int xPosition;
	private int yPosition;
	private Type initialType, type;

	/**
	* The visual type used if none is specified.  Currently this is DefaultVertexType1.
	*/
	public static final Type DEFAULT_TYPE = new DefaultVertexType1();

	/**
	 * @param vertex
	 */
	public PathwayVertex(PathwayVertex vertex)
	{
		this.graph = vertex.getGraph();
		this.id = vertex.getID();
		obj = vertex.getDataClass();
		this.dataClass = vertex.getDataClass();

		this.xPosition = vertex.getXPosition();
		this.yPosition = vertex.getYPosition();
		this.onLink = vertex.getOnLink();
		this.onLinkPosition = vertex.getOnLinkPosition();

		initialType = vertex.getInitialType();
		String label = vertex.getLabel();
		labeler = new LabeledElementImp(label);
		renderer =
			new VertexRenderer((VertexRenderer)vertex.getRenderer(), this);
		this.type = renderer.getType();
		rescale();
	}

	public PathwayVertex(
		Graph graph,
		int id,
		DataClass dcl,
		int xPosition,
		int yPosition)
	{
		this(graph, id, dcl, null, xPosition, yPosition, -1, -1);
	}

	public PathwayVertex(
		Graph graph,
		int id,
		DataClass dcl,
		Type type,
		int xPosition,
		int yPosition)
	{
		this(graph, id, dcl, type, xPosition, yPosition, -1, -1);
	}

	public PathwayVertex(
		Graph graph,
		int id,
		DataClass dcl,
		int xPosition,
		int yPosition,
		int onLink,
		double onLinkPosition)
	{
		this(
			graph,
			id,
			dcl,
			null,
			xPosition,
			yPosition,
			onLink,
			onLinkPosition);
	}

	public PathwayVertex(
		Graph graph,
		int id,
		DataClass dataClass,
		Type type,
		int xPosition,
		int yPosition,
		int onLink,
		double onLinkPosition)
	{
		this.graph = graph;
		this.id = id;
		obj = dataClass;
		this.dataClass = dataClass;
		this.type = type;
		this.xPosition = xPosition;
		this.yPosition = yPosition;
		this.onLink = onLink;
		this.onLinkPosition = onLinkPosition;

		initialType =
			(dataClass.getStoredType() != null)
				? dataClass.getStoredType()
				: DEFAULT_TYPE;

		if (this.type == null)
			this.type = initialType;

		String label = (String)dataClass.get("Name");
		labeler = new LabeledElementImp(label);
		renderer = new VertexRenderer(this);
		rescale();
	}

	public Type getInitialType()
	{
		return initialType;
	}

	/* (non-Javadoc)
	 * @see vp.graph.LabeledElement#getLabel()
	 */
	public String getLabel()
	{
		return labeler.getLabel();
	}
	/* (non-Javadoc)
	 * @see vp.graph.LabeledElement#hasLabel()
	 */
	public boolean hasLabel()
	{
		return labeler.hasLabel();
	}
	/* (non-Javadoc)
	 * @see vp.graph.LabeledElement#setLabel(java.lang.String)
	 */
	public void setLabel(String label)
	{
		labeler.setLabel(label);
	}

	/**
	 * @return
	 */
	public LabeledElementImp getLabeler()
	{
		return labeler;
	}

	/**
	 * @return
	 */
	public Renderer getRenderer()
	{
		return renderer;
	}

	/**
	 * @return
	 */
	public Type getType()
	{
		return type;
	}

	/**
	 * @return
	 */
	public int getXPosition()
	{
		return xPosition;
	}

	/**
	 * @return
	 */
	public int getYPosition()
	{
		return yPosition;
	}

	/**
	 * @param type
	 */
	public void setType(Type type)
	{
		this.type = type;
		renderer.setType(type);
	}

	/**
	 * @param i
	 */
	public void setXPosition(int i)
	{
		xPosition = i;
	}

	/**
	 * @param i
	 */
	public void setYPosition(int i)
	{
		yPosition = i;
	}

	/**
	 * @return
	 */
	public int getOnLink()
	{
		return onLink;
	}

	/**
	 * @return
	 */
	public double getOnLinkPosition()
	{
		return onLinkPosition;
	}

	/**
	 * @param i
	 */
	public void setOnLink(int i)
	{
		onLink = i;
	}

	/**
	 * @param i
	 */
	public void setOnLinkPosition(int i)
	{
		onLinkPosition = i;
	}

	/* (non-Javadoc)
	 * @see vp.visual.LabeledRenderer#getFillColor()
	 */
	public Color getFillColor()
	{
		return ((VertexRenderer)renderer).getFillColor();
	}

	/* (non-Javadoc)
	 * @see vp.visual.LabeledRenderer#getFont()
	 */
	public Font getFont()
	{
		return ((VertexRenderer)renderer).getFont();
	}

	/* (non-Javadoc)
	 * @see vp.visual.LabeledRenderer#getFontColor()
	 */
	public Color getFontColor()
	{
		return ((VertexRenderer)renderer).getFontColor();
	}

	/* (non-Javadoc)
	 * @see vp.visual.LabeledRenderer#getFontMetrics()
	 */
	public FontMetrics getFontMetrics()
	{
		return ((VertexRenderer)renderer).getFontMetrics();
	}

	/* (non-Javadoc)
	 * @see vp.visual.LabeledRenderer#rescale()
	 */
	public void rescale()
	{
		((VertexRenderer)renderer).rescale();
	}

	/* (non-Javadoc)
	 * @see vp.visual.LabeledRenderer#setFillColor(java.awt.Color)
	 */
	public void setFillColor(Color fillColor)
	{
		((VertexRenderer)renderer).setFillColor(fillColor);
	}

	/* (non-Javadoc)
	 * @see vp.visual.LabeledRenderer#setFont(java.awt.Font)
	 */
	public void setFont(Font font)
	{
		((VertexRenderer)renderer).setFont(font);
	}

	/* (non-Javadoc)
	 * @see vp.visual.LabeledRenderer#setFontColor(java.awt.Color)
	 */
	public void setFontColor(Color fontColor)
	{
		((VertexRenderer)renderer).setFontColor(fontColor);
	}

	/* (non-Javadoc)
	 * @see vp.visual.LabeledRenderer#setLocation(double, double)
	 */
	public void setLocation(double x, double y)
	{
		((VertexRenderer)renderer).setLocation(x, y);
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#containsPoint(int, int)
	 */
	public boolean containsPoint(int x, int y)
	{
		return ((VertexRenderer)renderer).containsPoint(x, y);
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getBounds()
	 */
	public Rectangle getBounds()
	{
		return ((VertexRenderer)renderer).getBounds();
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getBounds2D()
	 */
	public Rectangle2D getBounds2D()
	{
		return ((VertexRenderer)renderer).getBounds2D();
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getGeneralPath()
	 */
	public GeneralPath getGeneralPath()
	{
		return ((VertexRenderer)renderer).getGeneralPath();
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getOutlineColor()
	 */
	public Color getOutlineColor()
	{
		return ((VertexRenderer)renderer).getOutlineColor();
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getPainter()
	 */
	public Painter getPainter()
	{
		return ((VertexRenderer)renderer).getPainter();
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getTexturePaint()
	 */
	public TexturePaint getTexturePaint()
	{
		return ((VertexRenderer)renderer).getTexturePaint();
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#paint(java.awt.Graphics2D)
	 */
	public void paint(Graphics2D g2)
	{
		((VertexRenderer)renderer).paint(g2);
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setGeneralPath(java.awt.geom.GeneralPath)
	 */
	public void setGeneralPath(GeneralPath path)
	{
		((VertexRenderer)renderer).setGeneralPath(path);
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setOutlineColor(java.awt.Color)
	 */
	public void setOutlineColor(Color outlineColor)
	{
		((VertexRenderer)renderer).setOutlineColor(outlineColor);
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setPainter(vp.visual.paint.Painter)
	 */
	public void setPainter(Painter painter)
	{
		((VertexRenderer)renderer).setPainter(painter);
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setTexturePaint(java.awt.TexturePaint)
	 */
	public void setTexturePaint(TexturePaint tp)
	{
		((VertexRenderer)renderer).setTexturePaint(tp);
	}

	/**
	 * @return
	 */
	public String getShape()
	{
		return ((VertexRenderer)renderer).getShape();
	}

	/**
	 * @param shape
	 */
	public void setShape(String shape)
	{
		((VertexRenderer)renderer).setShape(shape);
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getDataClass()
	 */
	public DataClass getDataClass()
	{
		return dataClass;
	}

	/* (non-Javadoc)
	 * @see vp.graph.PathwayElement#setRenderer(vp.visual.Renderer)
	 */
	public void setRenderer(Renderer renderer)
	{
		this.renderer = renderer;
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setDataClass(vp.graph.DataClass)
	 */
	public void setDataClass(DataClass dataClass)
	{
		this.dataClass = dataClass;
		String label = (String)dataClass.get("Name");
		labeler.setLabel(label);
		rescale();
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getElement()
	 */
	public PathwayElement getElement()
	{
		return renderer.getElement();
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#setElement(vp.graph.PathwayElement)
	 */
	public void setElement(PathwayElement element)
	{
		renderer.setElement(element);
	}

	/* (non-Javadoc)
	 * @see vp.graph.PathwayElement#getElementType()
	 */
	public int getElementType()
	{
		return DBConstants.VERTEX;
	}
}
