package vp.main;

import vp.graph.PathwayGraph;
import vp.gui.components.VPPanel;
import vp.util.ChangeModeEvent;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

/**
* A panel that holds the visual graph.  It is then stored in the <code>GraphScrollPane</code> which is in turn
* placed in the <code>GraphEditor</code> and finally in the <code>MainFrame</code>.  It also acts as an interface for 
* the various UI modes(states), forwarding mouse and key events to the proper mode.
*/
public class GraphPanel
	extends VPPanel
	implements MouseListener, MouseMotionListener, KeyListener
{
	private PathwayGraph pGraph;
	private GraphPanelMode gpm;
	private MainFrame parent;
	private GraphScrollPane gpContainer;
    private boolean firstMove;

	/**
	* Creates a graph panel using the specified values.  The background is set to white by default and the UI mode is 
	* set to view.
	*/
	public GraphPanel(MainFrame parent, GraphPanelMode gpm)
	{
		this.parent = parent;
		this.gpm = gpm;
		gpm.setGraphPanel(this);
		setLayout(null);
		setViewMode();
		setBackground(Color.white);
		addMouseListener(this);
		addMouseMotionListener(this);
        firstMove = true;
    }

	/**
	* Returns the current UI mode for the graph panel.  The mode is set with the "setXXXMode()" method.  Current 
	* available modes are: <br>
	* 1. View - Shows the graph in its final form with tooltips and links.  It cannot be modified in this mode.	<br>
	* 2. Edit - Existing elements of the graph can be moved and modified by the user. <br>
	* 3. Vertex - The user can add vertices by clicking their location on the graph. <br>
	* 4. Edge - The user can add edges by clicking their location on the graph. <br>
	*
	* @see #setViewMode()
	* @see #setEditMode()
	* @see #setVertexMode()
	* @see #setEdgeMode()
	*/
	public GraphPanelMode getCurrentMode()
	{
		return gpm;
	}

	/**
	* Returns the scroll pane that contains the graph panel.
	*
	* @see #setGPContainer(GraphScrollPane gpContainer)
	*/
	public GraphScrollPane getGPContainer()
	{
		return gpContainer;
	}

	/**
	* Returns the main application window to which the graph panel belongs.
	*/
	public MainFrame getMainFrame()
	{
		return parent;
	}

	/**
	* Returns the visual graph contained within the graph panel.
	*
	* @see #setVisualGraph(VisualGraph vGraph)
	*/
	public PathwayGraph getPathwayGraph()
	{
		return pGraph;
	}

	/**
	* Forwards the key event to the appropriate UI mode.
	*/
	public void keyPressed(KeyEvent e)
	{
		gpm = gpm.keyPressed(e);
	}

	/**
	* Forwards the key event to the appropriate UI mode.
	*/
	public void keyReleased(KeyEvent e)
	{
		gpm = gpm.keyReleased(e);
	}

	/**
	* Forwards the key event to the appropriate UI mode.
	*/
	public void keyTyped(KeyEvent e)
	{
		gpm = gpm.keyTyped(e);
	}

	/**
	* Forwards the mouse event to the appropriate UI mode.
	*/
	public void mouseClicked(MouseEvent e)
	{
		gpm = gpm.mouseClicked(e);
	}

	/**
	* Forwards the mouse event to the appropriate UI mode.
	*/
	public void mouseDragged(MouseEvent e)
	{
		gpm = gpm.mouseDragged(e);
	}

	/**
	* Forwards the mouse event to the appropriate UI mode.
	*/
	public void mouseEntered(MouseEvent e)
	{
		gpm = gpm.mouseEntered(e);
	}

	/**
	* Forwards the mouse event to the appropriate UI mode.
	*/
	public void mouseExited(MouseEvent e)
	{
		gpm = gpm.mouseExited(e);
	}

	/**
	* Forwards the mouse event to the appropriate UI mode.
	*/
	public void mouseMoved(MouseEvent e)
	{
		gpm.mouseMoved(e);
	}

	/**
	* Forwards the mouse event to the appropriate UI mode.
	*/
	public void mousePressed(MouseEvent e)
	{
		gpm = gpm.mousePressed(e);
	}

	/**
	* Forwards the mouse event to the appropriate UI mode.
	*/
	public void mouseReleased(MouseEvent e)
	{
		gpm = gpm.mouseReleased(e);
	}

	/**
	* Delegates responsibility for painting the graph.
	*/
	public void paintComponent(Graphics g)
	{
		Dimension vGraphDim;
		Graphics2D g2;
		int width, height;

		vGraphDim = pGraph.getMaxSize();
		if (vGraphDim.width > gpContainer.getSize().width)
			width = vGraphDim.width;
		else
			width = gpContainer.getSize().width;

		if (vGraphDim.height > gpContainer.getSize().height)
			height = vGraphDim.height;
		else
			height = gpContainer.getSize().height;

		setSize(width, height);
		setPreferredSize(new Dimension(width, height));

		revalidate();
		super.paintComponent(g);
		g2 = (Graphics2D)g;

		gpm.paint(g2);
	}

	/**
	* Forwards the request to change mode to the appropriate UI mode.
	*/
	public GraphPanelMode processChangeModeEvent(ChangeModeEvent cme)
	{
		gpm = gpm.recommendMode(cme);
		return gpm;
	}

	/**
	* Sets the visual graph contained within the graph panel to the one specified.
	*
	* @see #getVisualGraph()
	*/
	public void setPathwayGraph(PathwayGraph pGraph)
	{
		if (pGraph != null)
			pGraph.removeContainer(this);
		this.pGraph = pGraph;
        gpm.setPathwayGraph(pGraph);
		pGraph.addContainer(this);
		repaint();
	}

	/**
	* Sends a request to change the UI mode to "edge" mode.
	*
	* @see #getCurrentMode()
	*/
	public void setEdgeMode()
	{
		gpm =
			processChangeModeEvent(
				new ChangeModeEvent(this, new GraphPanelEdgeMode(this)));
	}

	/**
	* Sends a request to change the UI mode to "edit" mode.
	*
	* @see #getCurrentMode()
	*/
	public void setEditMode()
	{
		gpm =
			processChangeModeEvent(
				new ChangeModeEvent(this, new GraphPanelEditMode(this)));
	}

	/**
	* Sets the scroll pane that contains the graph panel to the one specified.
	*
	* @see #getGPContainer()
	*/
	public void setGPContainer(GraphScrollPane gpContainer)
	{
		this.gpContainer = gpContainer;
	}

	/**
	* Sends a request to change the UI mode to "move" mode.
	* 
	* @see #getCurrentMode()
	*/
	public void setMoveMode()
	{
		gpm =
			processChangeModeEvent(
				new ChangeModeEvent(this, new GraphPanelMoveMode(this)));
	}

	/**
	* Sends a request to change the UI mode to "vertex" mode.
	*
	* @see #getCurrentMode()
	*/
	public void setVertexMode()
	{
		gpm =
			processChangeModeEvent(
				new ChangeModeEvent(this, new GraphPanelVertexMode(this)));
	}

	/**
	* Sends a request to change the UI mode to "view" mode.
	*
	* @see #getCurrentMode()
	*/
	public void setViewMode()
	{
		gpm =
			processChangeModeEvent(
				new ChangeModeEvent(this, new GraphPanelViewMode(this)));
	}
	/**
	 * @return
	 */
	public boolean isFirstMove()
	{
		return firstMove;
	}

	/**
	 * @param b
	 */
	public void setFirstMove(boolean b)
	{
		firstMove = b;
	}

}
