package vp.main;

import vp.graph.PathwayEdge;
import vp.graph.PathwayVertex;
import vp.util.EditEdgeList;
import vp.util.EditVertexList;
import vp.visual.AbstractRenderer;

import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.event.MouseEvent;
import java.util.ListIterator;

/**
* The UI mode for editing the visual graph.  In this mode, the user can select vertices or edges to modify.
*/
public class GraphPanelEditMode extends GraphPanelMode
{
	private PathwayVertex targetVertex;
	private PathwayEdge targetEdge;
	private Cursor prevCursor;
	private Cursor handCursor;
	private EditVertexList highlightedVertices;
	private EditEdgeList highlightedEdges;
	private boolean lock;

	/**
	* Constructs a UI mode object for the specified panel.
	*/
	public GraphPanelEditMode(GraphPanel gPanel)
	{
		super(gPanel);
		handCursor = new Cursor(Cursor.HAND_CURSOR);
	}

	/**
	* Constructs a UI mode object with no specific graph panel.
	*/
	public GraphPanelEditMode()
	{
		this(null);
	}

	/**
	* Changes the graph panel's cursor depending on whether the mouse enters or leaves a vertex or edge.
	*/
	public GraphPanelMode mouseMoved(MouseEvent e)
	{
		pGraph = gPanel.getPathwayGraph();
		targetVertex = pGraph.getSelectedVertex(e.getX(), e.getY());
		targetEdge = pGraph.getSelectedEdge(e.getX(), e.getY());

		if (targetVertex != null)
		{
			if (prevCursor == null)
				prevCursor = gPanel.getCursor();
			gPanel.setCursor(handCursor);
		}
		else if (targetEdge != null)
		{
			if (prevCursor == null)
				prevCursor = gPanel.getCursor();
			gPanel.setCursor(handCursor);
		}
		else
		{
			gPanel.setCursor(prevCursor);
			prevCursor = null;
		}

		return this;
	}

	/**
	* Highlights or unhighlights the pertinent elements depending on the user.  Also will call the popup edit menu
	* if this is the popup trigger.  Usually, though, the popup trigger is mouse released.
	*/
	public GraphPanelMode mousePressed(MouseEvent e)
	{
		pGraph = gPanel.getPathwayGraph();
		int prevX = e.getX();
		int prevY = e.getY();
		highlightedVertices = pGraph.getHighlightedVertices();
		highlightedEdges = pGraph.getHighlightedEdges();
		lock = pGraph.isLocked();
        
        //Unhighlight vertices if shift is not down and not right click.
		if (!e.isShiftDown()
			&& !lock
			&& ((e.getModifiers() & MouseEvent.BUTTON3_MASK)
				!= MouseEvent.BUTTON3_MASK))
		{
			pGraph.clearHighlightedElements();
			highlightedVertices.clear();
			highlightedEdges.clear();

			ListIterator iter = pGraph.getVertexIterator();
			while (iter.hasNext())
				 ((PathwayVertex)iter.next()).setTexturePaint(null);

			iter = pGraph.getEdgeIterator();
			while (iter.hasNext())
				 ((PathwayEdge)iter.next()).setTexturePaint(null);

			gPanel.repaint();
		}

		targetVertex = gPanel.getPathwayGraph().getSelectedVertex(prevX, prevY);

		//Highlight the clicked vertex.
		if (!lock
			&& targetVertex != null
			&& ((e.getModifiers() & MouseEvent.BUTTON3_MASK)
				!= MouseEvent.BUTTON3_MASK))
		{
			highlightedEdges.clear();
			ListIterator iter = pGraph.getEdgeIterator();
			while (iter.hasNext())
				 ((PathwayEdge)iter.next()).setTexturePaint(null);

			highlightedVertices.add(targetVertex);
			for (int i = 0; i < highlightedVertices.size(); i++)
			{
				highlightedVertices.getVertex(i).setTexturePaint(
					AbstractRenderer.DEFAULT_TEXTURE);
			}
			gPanel.repaint();
		}
		/*
		//Calls popup menu
		if (e.isPopupTrigger() && targetVertex != null)
			new EditPopupMenu(targetVertex, gPanel, e.getX(), e.getY());
		else if(e.isPopupTrigger() && targetEdge != null)
			new EditPopupMenu(targetEdge, gPanel, e.getX(), e.getY());
		*/
		targetEdge = pGraph.getSelectedEdge(prevX, prevY);

		//Highlight the clicked edge
		if (!lock
			&& targetEdge != null
			&& ((e.getModifiers() & MouseEvent.BUTTON3_MASK)
				!= MouseEvent.BUTTON3_MASK))
		{
			highlightedVertices.clear();
			ListIterator iter = pGraph.getVertexIterator();
			while (iter.hasNext())
				 ((PathwayVertex)iter.next()).setTexturePaint(null);

			highlightedEdges.add(targetEdge);
			for (int i = 0; i < highlightedEdges.size(); i++)
			{
				highlightedEdges.getEdge(i).setTexturePaint(
					AbstractRenderer.DEFAULT_TEXTURE);
			}
			gPanel.repaint();
		}

		return this;
	}

	/**
	* Highlights or unhighlights the pertinent elements depending on the user.  Also will call the popup edit menu
	* if this is the popup trigger.
	*/
	public GraphPanelMode mouseReleased(MouseEvent e)
	{
		/*
		//Calls popup menu
		if (e.isPopupTrigger() && targetVertex != null)
			new EditPopupMenu(targetVertex, gPanel, e.getX(), e.getY());
		else if(e.isPopupTrigger() && targetEdge != null)
			new EditPopupMenu(targetEdge, gPanel, e.getX(), e.getY());
		*/
		targetVertex = null;
		targetEdge = null;
		return this;
	}

	/**
	* Calls the visual graph's paint command.
	*/
	public void paint(Graphics2D g2)
	{
		gPanel.getPathwayGraph().paint(g2);
	}

}
