package vp.main;

import java.awt.Graphics2D;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

import vp.graph.PathwayGraph;
import vp.util.ChangeModeEvent;


/**
* An abstract superclass for UI interaction modes on the graph panel.   Current available modes are: <br>
* 1. View - Shows the graph in its final form with tooltips and links.  It cannot be modified in this mode.	<br>
* 2. Edit - Existing elements of the graph can be moved and modified by the user. <br>
* 3. Vertex - The user can add vertices by clicking their location on the graph. <br>
* 4. Edge - The user can add edges by clicking their location on the graph. <br>
*
* @see edu.yale.graph.visual.GraphPanelEdgeMode
* @see edu.yale.graph.visual.GraphPanelEditMode
* @see edu.yale.graph.visual.GraphPanelVertexMode
* @see edu.yale.graph.visual.GraphPanelViewMode
*/
public abstract class GraphPanelMode
{
	GraphPanel gPanel;
	PathwayGraph pGraph;
	MainFrame mainFrame;

	/**
	* Constructs a UI mode object for the specified panel.
	*/
	GraphPanelMode(GraphPanel gPanel)
	{
		this.gPanel = gPanel;
        
	}

	/**
	* Constructs a UI mode object with no specific graph panel.
	*/
	GraphPanelMode()
	{
	}

	/**
	* Returns the underlying graph panel.
	*/
	public GraphPanel getGraphPanel()
	{
		return gPanel;
	}
	
	/**
	* Forwards the key event to the appropriate UI mode.
	*/
	public GraphPanelMode keyPressed(KeyEvent e)
	{
		return this;
	}

	/**
	* Forwards the key event to the appropriate UI mode.
	*/
	public GraphPanelMode keyReleased(KeyEvent e)
	{
		return this;
	}

	/**
	* Forwards the key event to the appropriate UI mode.
	*/
	public GraphPanelMode keyTyped(KeyEvent e)
	{
		return this;
	}
	
	/**
	* Forwards the mouse event to the appropriate UI mode.
	*/
	public GraphPanelMode mouseClicked(MouseEvent e)
	{
		return this;
	}

	/**
	* Forwards the mouse event to the appropriate UI mode.
	*/
	public GraphPanelMode mouseDragged(MouseEvent e)
	{
		return this;
	}

	/**
	* Forwards the mouse event to the appropriate UI mode.
	*/
	public GraphPanelMode mouseEntered(MouseEvent e)
	{
		return this;
	}

	/**
	* Forwards the mouse event to the appropriate UI mode.
	*/
	public GraphPanelMode mouseExited(MouseEvent e)
	{
		return this;
	}
	
	/**
	* Forwards the mouse event to the appropriate UI mode.
	*/
	public GraphPanelMode mouseMoved(MouseEvent e)
	{
		return this;
	}

	/**
	* Forwards the mouse event to the appropriate UI mode.
	*/
	public GraphPanelMode mousePressed(MouseEvent e)
	{
		return this;
	}

	/**
	* Forwards the mouse event to the appropriate UI mode.
	*/
	public GraphPanelMode mouseReleased(MouseEvent e)
	{
		return this;
	}
	
	/**
	* Calls the paint methods of the appropriate UI mode.
	*/
	public abstract void paint(Graphics2D g2);
	
	/**
	* Requests the the current UI mode be changed to the one specified in the <code>ChangeModeEvent</code>.
	*/
	public GraphPanelMode recommendMode(ChangeModeEvent cme)
	{
		return cme.getRequestedMode();
	}

	/**
	* Sets the underlying graph panel to the one specified.
	*/
	public void setGraphPanel(GraphPanel gPanel)
	{
		this.gPanel = gPanel;
		this.pGraph = gPanel.getPathwayGraph();
        this.mainFrame = gPanel.getMainFrame();
	}
	/**
	 * @return
	 */
	public MainFrame getMainFrame()
	{
		return mainFrame;
	}

	/**
	 * @return
	 */
	public PathwayGraph getPathwayGraph()
	{
		return pGraph;
	}

	/**
	 * @param graph
	 */
	public void setPathwayGraph(PathwayGraph pGraph)
	{
		this.pGraph = pGraph;
	}

}
