package vp.util;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;

import vp.main.MainFrame;
/**
 * This is a variant of the SwingWorker
 * It works in conjunction with the GlassPane class
 * to allow users to execute timeconsuming task on a separate thread
 * The GlassPane addition can prevent users from executing another SwingWorker task
 * while one SwingWorker task is already executing
 *
 * @author Yexin Chen
 */
public abstract class VPSwingWorker
{
	private boolean repaint;
	private int timePer;
	private float v;
	private int min;
	private Thread thread;
	private MainFrame parent;
	private String task;
	private long startTime;
	private int max;
	/** 
	 * Class to maintain reference to current worker thread
	 * under separate synchronization control.
	 */
	private static class ThreadVar
	{
		private Thread thread;
		ThreadVar(Thread t)
		{
			thread = t;
		}
		synchronized Thread get()
		{
			return thread;
		}
		synchronized void clear()
		{
			thread = null;
		}
	}

	private ThreadVar threadVar;

	private GlassPane glassPane;
	private java.awt.Component aComponent;
	/**
	 * Start a thread that will call the <code>construct</code> method
	 * and then exit.
	 * @param aComponent a reference to the UI component that's directly using
	 * SwingWorker
	 */
	public VPSwingWorker(MainFrame parent, Component aComponent, String task)
	{
		this(parent, aComponent, task, 0, 95, 1, true);
	}

	public VPSwingWorker(
		MainFrame parent,
		Component aComponent,
		String task,
		int min,
		int max)
	{
		this(parent, aComponent, task, min, max, 1, true);
	}

	public VPSwingWorker(
		MainFrame parent,
		Component aComponent,
		String task,
		int min,
		int max,
		int timePer)
	{
		this(parent, aComponent, task, min, max, timePer, true);
	}

	public VPSwingWorker(
		MainFrame parent,
		Component aComponent,
		String task,
		int min,
		int max,
		boolean repaint)
	{
		this(parent, aComponent, task, min, max, 1, repaint);

	}

	public VPSwingWorker(
		MainFrame parent,
		Component aComponent,
		String task,
		int min,
		int max,
		int timePer,
		boolean repaint)
	{
		this.parent = parent;
		this.task = task;
		this.max = max;
		this.min = min;
		this.timePer = timePer;
		this.repaint = repaint;
		setAComponent(aComponent);

		final Runnable doFinished = new Runnable()
		{
			public void run()
			{
				finished();
			}
		};

		Runnable doConstruct = new Runnable()
		{
			public void run()
			{
				try
				{
					construct();
				}
				finally
				{
					threadVar.clear();
				}

				// Execute the doFinished runnable on the Swing dispatcher thread

				SwingUtilities.invokeLater(doFinished);
			}

		};

		// Group the new worker thread in the same group as the "spawner" thread
		Thread t =
			new Thread(Thread.currentThread().getThreadGroup(), doConstruct);
		threadVar = new ThreadVar(t);
	}
	/**
	 * Activate the capabilities of glasspane
	 * 
	 */
	private void activateGlassPane()
	{
		// Mount the glasspane on the component window
		GlassPane aPane = GlassPane.mount(getAComponent(), true);

		// keep track of the glasspane as an instance variable
		setGlassPane(aPane);

		if (getGlassPane() != null)
		{
			// Start interception UI interactions
			getGlassPane().setVisible(true);
		}
	}
	/** 
	 * Enable the glass pane (to disable unwanted UI manipulation), then spawn the non-UI logic on a separate thread.
	 */
	private void construct()
	{
		activateGlassPane();
		parent.getProgBar().setValue(min);
		parent.getProgBar().setVisible(true);
		parent.getAlertField().setVisible(true);
		parent.getAlertField().setText(StringTable.retrieve(task));
		int initDelay = (VPProperties.getTime(task) * timePer) / max;
		int delay = 15 * ((int) (initDelay / 15) + 1);
		v = (float)delay / initDelay;
		ActionListener listener = new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{

				if (parent.getProgBar().getValue() <= max)
					parent.getProgBar().setValue(
						parent.getProgBar().getValue() + (int)v);

			}
		};
		Timer timer = new Timer(delay, listener);
		timer.start();
		startTime = System.currentTimeMillis();
		try
		{
			doNonUI();
		}
		catch (RuntimeException e)
		{}
		long stopTime = System.currentTimeMillis();
		VPProperties.reportTime(task, (int) ((stopTime - startTime) / timePer));
		timer.stop();
	}
	/**
	 * Deactivate the glasspane
	 * 
	 */
	private void deactivateGlassPane()
	{
		if (getGlassPane() != null)
		{
			// Stop UI interception
			getGlassPane().setVisible(false);
		}
	}
	/**
	 * This method will be implemented by the inner class of SwingWorker
	 * It should only consist of the logic that's unrelated to UI
	 *
	 * @throws java.lang.RuntimeException thrown if there are any errors in the non-ui logic
	 */
	protected abstract void doNonUI() throws RuntimeException;
	/**
	 * This method will be implemented by the inner class of SwingWorker
	 * It should only consist of the logic that's related to UI updating, after
	 * the doNonUILogic() method is done. 
	 *
	 * @throws java.lang.RuntimeException thrown if there are any problems executing the ui update logic
	 */
	protected abstract void doUI() throws RuntimeException;
	/**
	 * Called on the event dispatching thread (not on the worker thread)
	 * after the <code>construct</code> method has returned.
	 */
	protected void finished()
	
	{
		try
		{
			if (repaint)
				deactivateGlassPane();
			doUI();
		}
		catch (RuntimeException e)
		{
			e.printStackTrace();
		}
		finally
		{
			// Allow original component to get the focus
			if (getAComponent() != null)
			{
				getAComponent().requestFocus();
			}
			if (repaint)
			{
				parent.getAlertField().setVisible(false);
				parent.getProgBar().setVisible(false);
			}
		}
	}
	/**
	 * Getter method
	 * 
	 * @return java.awt.Component
	 */
	protected Component getAComponent()
	{
		return aComponent;
	}
	/**
	 * Getter method
	 * 
	 * @return GlassPane
	 */
	protected GlassPane getGlassPane()
	{
		return glassPane;
	}
	/**
	 * A new method that interrupts the worker thread.  Call this method
	 * to force the worker to stop what it's doing.
	 */
	public void interrupt()
	{
		Thread t = threadVar.get();
		if (t != null)
		{
			t.interrupt();
		}
		threadVar.clear();
	}
	/**
	 * Setter method
	 * 
	 * @param newAComponent java.awt.Component
	 */
	protected void setAComponent(Component newAComponent)
	{
		aComponent = newAComponent;
	}
	/**
	 * Setter method
	 * 
	 * @param newGlassPane GlassPane
	 */
	protected void setGlassPane(GlassPane newGlassPane)
	{
		glassPane = newGlassPane;
	}
	/**
	 * Start the worker thread.
	 */
	public void start()
	{
		Thread t = threadVar.get();
		if (t != null)
		{
			t.start();
		}
	}
}
