package vp.util.geom;

import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import vp.graph.PathwayEdge;
import vp.graph.PathwayVertex;

public class Geometry
{
	private static final int BEZIER_POINTS = 20;

	private Geometry()
	{}

	public static Equation getEquation(Point2D.Double p1, Point2D.Double p2)
	{
		return new Equation(p1, p2);
	}

	public static Point2D.Double getMidpoint(
		Point2D.Double p1,
		Point2D.Double p2)
	{
		double x = (p1.x + p2.x) / 2;
		double y = (p1.y + p2.y) / 2;
		return new Point2D.Double(x, y);
	}

	public static double getXSpan(Point2D.Double p1, Point2D.Double p2)
	{
		return Math.max(p1.x, p2.x) - Math.min(p1.x, p2.x);
	}

	public static double getYSpan(Point2D.Double p1, Point2D.Double p2)
	{
		return Math.max(p1.y, p2.y) - Math.min(p1.y, p2.y);
	}

	public static Point2D.Double[] getBezier(
		Point2D.Double start,
		Point2D.Double control,
		Point2D.Double end)
	{
		Point2D.Double[] bezier = new Point2D.Double[BEZIER_POINTS];
		double percent = 1f / BEZIER_POINTS;

		for (int i = 0; i < bezier.length; i++)
		{
			double t = percent * i;
			double p00 = start.x + ((control.x - start.x) * t);
			double p01 = start.y + ((control.y - start.y) * t);
			double p10 = control.x + ((end.x - control.x) * t);
			double p11 = control.y + ((end.y - control.y) * t);
			double x = p00 + ((p10 - p00) * t);
			double y = p01 + ((p11 - p01) * t);
			bezier[i] = new Point2D.Double(x, y);
		}
		return bezier;
	}

	public static Point2D.Double getPtOnPerp(
		Point2D.Double p1,
		Point2D.Double p2,
		double percent,
		int maxDistance,
		double position)
	{
		double spacing, x2, y2;
		Line2D.Double line = new Line2D.Double(p1, p2);
		Point2D.Double mid = getMidpoint(p1, p2);
		Equation eq = getEquation(p1, p2);
		double xSpan = getXSpan(p1, p2);
		double ySpan = getYSpan(p1, p2);
		if (xSpan > ySpan)
		{
			spacing = xSpan * percent;
			x2 = Math.min(p1.x, p2.x) + spacing;
			y2 = eq.getYValue(x2);
		}
		else
		{
			spacing = ySpan * percent;
			y2 = Math.min(p1.y, p2.y) + spacing;
			x2 = eq.getXValue(y2);
		}

		AffineTransform at = new AffineTransform();
		double distance = Point2D.distance(p1.x, p1.y, p2.x, p2.y);
		if (distance > maxDistance)
		{
			double scaleFactor = 1 / (distance / maxDistance);
			double transFactor = (1 / scaleFactor) - 1;
			at.scale(scaleFactor, scaleFactor);
			at.translate(x2 * transFactor, y2 * transFactor);
		}
		at.rotate(Math.PI / 2, mid.x, mid.y);

		GeneralPath oldPath = new GeneralPath(line);
		double[] prev = new double[2];
		double[] curr = new double[2];
		double[] edge = new double[6];
		Line2D.Double line2 = null;
		PathIterator pi = oldPath.getPathIterator(at);
		while (!pi.isDone())
		{
			prev[0] = curr[0];
			prev[1] = curr[1];
			int seg = pi.currentSegment(edge);
			curr[0] = edge[0];
			curr[1] = edge[1];
			if (seg == PathIterator.SEG_LINETO)
			{
				line2 = new Line2D.Double(prev[0], prev[1], curr[0], curr[1]);
				break;
			}
			pi.next();
		}

		if (position < 0)
			return new Point2D.Double((double)line2.x1, (double)line2.y1);
		else
			return new Point2D.Double((double)line2.x2, (double)line2.y2);
	}

	public static double logg(double value)
	{
		double returnValue = 0;
		if (value == 0)
			return -1;
		value = Math.abs(value);
		returnValue = Math.log(value) / Math.log(10);
		if (returnValue < -1)
			return -1;
		else if (returnValue > 1)
			return 1;
		else
			return returnValue;
	}

	/**
	 * @param edge
	 * @param linkRatio
	 * @return
	 */
	public static Point2D.Double getLinkPosition(PathwayEdge edge, double linkRatio)
	{
        PathwayVertex vertA = (PathwayVertex)edge.getVertexA();
        PathwayVertex vertB = (PathwayVertex)edge.getVertexB();
        int vax = vertA.getXPosition();
        int vbx = vertB.getXPosition();
        int vay = vertA.getYPosition();
        int vby = vertB.getYPosition();
        Point2D.Double va = new Point2D.Double(vax, vay);
        Point2D.Double vb = new Point2D.Double(vbx, vby);

        if (edge.getCurvature() == 0)
        {

            double x =
                ((Math.max(vax, vbx) - Math.min(vax, vbx)) * linkRatio)
                    + Math.min(vax, vbx);
            Equation eq = Geometry.getEquation(va, vb);
            double y;
            if (vax == vbx)
                y =
                    ((Math.max(vay, vby) - Math.min(vay, vby)) * linkRatio)
                        + Math.min(vay, vby);
            else
                y = (double)eq.getYValue(x);
            return new Point2D.Double(x, y);
        }
        else
            return CurvePath.getLinkPosition(
                new Point2D.Double(vax, vay),
                new Point2D.Double(vbx, vby),
                linkRatio);
	}
}
