package vp.util.xml;

import vp.graph.DataClass;
import vp.graph.GraphDataClass;
import vp.graph.PathwayDecoration;
import vp.graph.PathwayEdge;
import vp.graph.PathwayGraph;
import vp.graph.PathwayVertex;
import vp.visual.type.Type;

import java.io.File;
import java.util.ListIterator;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class GraphToXML
{
	private static Document doc;

	private GraphToXML()
	{}

	public static Document generateDocument(PathwayGraph pGraph)
	{
		writeDocument(pGraph);
		return doc;
	}

	public static void saveXMLFile(PathwayGraph pGraph, File file)
	{
		writeDocument(pGraph);
		saveDocument(file);
	}

	private static void saveDocument(File file)
		throws TransformerFactoryConfigurationError
	{
		try
		{
			TransformerFactory tf = TransformerFactory.newInstance();
			Transformer t = tf.newTransformer();
			DOMSource source = new DOMSource(doc);
			StreamResult result = new StreamResult(file);
			t.setOutputProperty(OutputKeys.INDENT, "yes");
			t.setOutputProperty(
				"{http://xml.apache.org/xslt}indent-amount",
				"2");
			t.setOutputProperty(OutputKeys.DOCTYPE_SYSTEM, "vpxml.dtd");
			t.transform(source, result);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	private static void writeDocument(PathwayGraph pGraph)
	{
		doc = XMLFactory.getSaveDocument();
		Element root = doc.getDocumentElement();
		GraphDataClass pw = pGraph.getGraphDataClass();
		root.setAttribute("id", String.valueOf(pw.get("id")));
		root.setAttribute("name", (String)pw.get("Name"));
		root.setAttribute("organism", (String)pw.get("Organism"));
		root.setAttribute(
			"annotation",
			((String)pw.get("Annotation") == null)
				? ""
				: (String)pw.get("Annotation"));
		ListIterator iter = pGraph.getVertexIterator();
		while (iter.hasNext())
		{
			try
			{
				PathwayVertex pv = (PathwayVertex)iter.next();
				writeVertex(root, pv);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}

		ListIterator edgeIter = pGraph.getEdgeIterator();

		while (edgeIter.hasNext())
		{
			try
			{
				PathwayEdge pe = (PathwayEdge)edgeIter.next();

				writeEdge(root, pe);
			}

			catch (Exception xe)
			{
				xe.printStackTrace();
			}
		}
	}

	private static void writeEdge(Element root, PathwayEdge pe)
	{
		Element linkNode = doc.createElement("LINK");
		linkNode.setAttribute("id", String.valueOf(pe.getID()));
		Element vertexA = doc.createElement("vertexA");
		Element vertexB = doc.createElement("vertexB");
		Element direction = doc.createElement("direction");
		Element curvature = doc.createElement("curvature");
		Element linkColor = doc.createElement("linkColor");
		Element solid = doc.createElement("solid");

		add(root, linkNode);
		add(linkNode, vertexA);
		add(linkNode, vertexB);
		add(linkNode, direction);
		add(linkNode, curvature);
		add(linkNode, linkColor);
		add(linkNode, solid);

		add(vertexA, String.valueOf(pe.getVertexA().getID()));
		add(vertexB, String.valueOf(pe.getVertexB().getID()));
		add(direction, pe.getDirection());
		add(curvature, String.valueOf(pe.getCurvature()));
		add(linkColor, String.valueOf(pe.getOutlineColor().getRGB()));
		add(solid, pe.getShape() == "Dotted" ? "N" : "Y");

		ListIterator decIter = pe.getDecorationIterator();

		while (decIter.hasNext())
		{
			PathwayDecoration pd = (PathwayDecoration)decIter.next();
			writeDecoration(linkNode, pd);
		}
	}

	private static void writeDecoration(Element linkNode, PathwayDecoration pd)
	{
		if (pd.getDataClass().getClassName().equals("Enzyme"))
		{
			DataClass pdDataClass = pd.getDataClass();

			Element decNode = doc.createElement("DECORATION");
			decNode.setAttribute("id", String.valueOf(pd.getID()));
            decNode.setAttribute("className", pdDataClass.getClassName());
			Element decDataClass = doc.createElement("decDataClass");
			decDataClass.setAttribute("id", (String)pdDataClass.get("id"));
            decDataClass.setAttribute("altName", (String)pdDataClass.get("ECNumber"));
			decDataClass.setAttribute("name", (String)pdDataClass.get("Name"));
			decDataClass.setAttribute(
				"annotation",
				((String)pdDataClass.get("Annotation") == null)
					? ""
					: (String)pdDataClass.get("Annotation"));
			//Element enzymeName = doc.createElement("enzyme-name");
			Element decShape = doc.createElement("decShape");
			Element decOutlineColor = doc.createElement("decOutlineColor");
			Element decFillColor = doc.createElement("decFillColor");
			Element decFont = doc.createElement("decFont");
			Element decFontSize = doc.createElement("decFontSize");
			Element decFontColor = doc.createElement("decFontColor");

			add(linkNode, decNode);
			add(decNode, decDataClass);
			//add(enzyme, enzymeName);
			add(decNode, decShape);
			add(decNode, decOutlineColor);
			add(decNode, decFillColor);
			add(decNode, decFont);
			add(decNode, decFontSize);
			add(decNode, decFontColor);

			//add(enzymeName, vdEnzyme.getName());
			add(decShape, pd.getShape());
			add(decOutlineColor, String.valueOf(pd.getOutlineColor().getRGB()));
			add(decFillColor, String.valueOf(pd.getFillColor().getRGB()));
			String fontName = pd.getFont().getFontName();
			int fontEnd =
				(fontName.indexOf(".") == -1)
					? fontName.length()
					: fontName.indexOf(".");
			add(
				decFont,
				fontName.substring(0, fontEnd)
					+ "_"
					+ String.valueOf(pd.getFont().getStyle()));
			add(decFontSize, String.valueOf(pd.getFont().getSize()));
			add(decFontColor, String.valueOf(pd.getFontColor().getRGB()));

			if (pd.getDataClass().get("Expression") != null)
			{
				Element expressionValue = doc.createElement("expression-value");
				add(decNode, expressionValue);
				add(
					expressionValue,
					(String)pd.getDataClass().get("Expression"));
			}
		}
	}

	private static void writeVertex(Element root, PathwayVertex pv)
	{
		Type pvType = pv.getType();
		DataClass pvDataClass = pv.getDataClass();

		Element vertexNode = doc.createElement("VERTEX");
		vertexNode.setAttribute("id", String.valueOf(pv.getID()));
		Element vertDataClass = doc.createElement("vertDataClass");
		vertDataClass.setAttribute("id", (String)pvDataClass.get("id"));
		vertDataClass.setAttribute("name", (String)pvDataClass.get("Name"));
		vertDataClass.setAttribute(
			"annotation",
			((String)pvDataClass.get("Annotation") == null)
				? ""
				: (String)pvDataClass.get("Annotation"));

		Element xPosition = doc.createElement("xposition");
		Element yPosition = doc.createElement("yposition");
		Element vertShape = doc.createElement("vertShape");
		Element vertOutlineColor = doc.createElement("vertOutlineColor");
		Element vertFillColor = doc.createElement("vertFillColor");
		Element vertFont = doc.createElement("vertFont");
		Element vertFontSize = doc.createElement("vertFontSize");
		Element vertFontColor = doc.createElement("vertFontColor");
		Element onLink = doc.createElement("on-link");
		Element onLinkPosition = doc.createElement("on-link-position");

		add(root, vertexNode);
		add(vertexNode, vertDataClass);
		//				add(compound, compoundName);
		//				add(compound, compoundComments);
		add(vertexNode, xPosition);
		add(vertexNode, yPosition);
		add(vertexNode, vertShape);
		add(vertexNode, vertOutlineColor);
		add(vertexNode, vertFillColor);
		add(vertexNode, vertFont);
		add(vertexNode, vertFontSize);
		add(vertexNode, vertFontColor);
		add(vertexNode, onLink);
		add(vertexNode, onLinkPosition);

		//add(compoundName, vvCompound.getName());

		add(xPosition, String.valueOf(pv.getXPosition()));
		add(yPosition, String.valueOf(pv.getYPosition()));
		add(vertShape, pvType.getShape());
		add(
			vertOutlineColor,
			String.valueOf(pvType.getOutlineColor().getRGB()));
		add(vertFillColor, String.valueOf(pvType.getFillColor().getRGB()));
		add(
			vertFont,
			pvType.getFont().getFontName()
				+ "_"
				+ String.valueOf(pvType.getFont().getStyle()));
		add(vertFontSize, String.valueOf(pvType.getFont().getSize()));
		add(vertFontColor, String.valueOf(pvType.getFontColor().getRGB()));
		add(onLink, String.valueOf(pv.getOnLink()));
		add(onLinkPosition, String.valueOf(pv.getOnLinkPosition()));
	}

	private static void add(Element parent, Element child)
	{
		parent.appendChild(child);
	}

	private static void add(Element parent, String child)
	{
		parent.appendChild(doc.createTextNode(child));
	}
}
