package vp.visual;

import vp.graph.DataClass;
import vp.graph.PathwayElement;
import vp.util.GraphicsFactory;
import vp.visual.paint.Painter;
import vp.visual.type.Type;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.TexturePaint;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;

/**
* An abstract implementation of <code>VisualGraphElement</code>.   This class contains default implementations for 
* methods shared by <code>VisualVertex</code>, <code>VisualDecoration</code> and <code>VisualEdge</code>.  It is 
* extended by each <code>VisualEdge</code>.  The other two also inherit from <code>AbstractLabeledVisualGraphElement
* </code>.
*
* @see edu.yale.graph.visual.VisualGraphElement
* @see edu.yale.graph.visual.VisualVertex
* @see edu.yale.graph.visual.VisualDecoration
* @see edu.yale.graph.visual.VisualEdge
*/
public abstract class AbstractRenderer implements Renderer
{
	TexturePaint texturePaint = null;
	public static TexturePaint DEFAULT_TEXTURE =
		GraphicsFactory.getTexturePaint();
	Type type;
	GeneralPath drawPath;
	protected Painter painter;
	DataClass dataClass;
	PathwayElement element;

	/** 
	* Returns true if the specified point is contained within the boundaries of the visual element, false otherwise.
	*
	* @param x	The x coordinate of the point.
	* @param y	The y coordinate of the point.
	*/
	public boolean containsPoint(int x, int y)
	{
		return drawPath.contains(x, y);
	}

	/**
	* Returns a rectangle expressing the boundaries of the visual element.
	*
	* @see #getBounds2D()
	*/
	public Rectangle getBounds()
	{
		return drawPath.getBounds();
	}

	/**
	* Returns a 2D rectangle expressing the boundaries of the visual element.
	*
	* @see #getBounds()
	*/
	public Rectangle2D getBounds2D()
	{
		return drawPath.getBounds2D();
	}

	/**
	* Returns the geometric draw path of the visual element.
	*
	* @see #setGeneralPath(GeneralPath generalPath)
	*/
	public GeneralPath getGeneralPath()
	{
		return drawPath;
	}

	/**
	* Returns the outline color of the visual element.  Black is the default.
	*
	* @see #setOutlineColor(Color outlineColor)
	*/
	public Color getOutlineColor()
	{
		return type.getOutlineColor();
	}

	/**
	* Returns the <code>Painter</code> object used to paint the visual element.
	*
	* @see #setPainter(Painter painter)
	*/
	public Painter getPainter()
	{
		return painter;
	}

	/**
	* Returns a string describing the shape of the current type.  Currently available shapes are: "Ellipse", 
	* "Rectangle", "Rounded Rectangle", "Line" and "Dotted Line".
	*
	* @see #setShape(String shape)
	*/
	public String getShape()
	{
		return type.getShape();
	}

	/**
	* Returns the textured pattern used to draw the visual element when it is highlighted or <code>null</code> if not 
	* set.  The default setting is off until the element in question is highlighted by the user.
	*
	* @see #setTexturePaint()
	* @see #setTexturePaint(TexturePaint texturePaint)
	*/
	public TexturePaint getTexturePaint()
	{
		return texturePaint;
	}

	/**
	* Returns the visual type of the element.
	*
	* @see #setType(Type type)
	*/
	public Type getType()
	{
		return type;
	}

	/**
	* Returns a full-length descriptive name for the current type of the visual element.
	*
	* @see #setTypeName(String name)
	*/
	public String getTypeName()
	{
		return type.getName();
	}

	/**
	* Returns a brief name for the current type of the visual element.  This is the name under which a user-defined 
	* type will be saved.
	*
	* @see #setTypeShortName(String shortName)
	*/
	public String getTypeShortName()
	{
		return type.getShortName();
	}

	/**
	* Notifies the element's painter to paint the element.
	*/
	public void paint(Graphics2D g2)
	{
		painter.paint(element, g2);
	}

	/**
	* Sets the geometric draw path of the visual element to the value specified.
	* 
	* @see #getGeneralPath()
	*/
	public void setGeneralPath(GeneralPath drawPath)
	{
		this.drawPath = drawPath;
	}

	/**
	* Sets the outline color of the visual element to the value specified.
	*
	* @see #getOutlineColor()
	*/
	public void setOutlineColor(Color outlineColor)
	{
		type.setOutlineColor(outlineColor);
	}

	/**
	* Sets the <code>Painter</code> object used by the element to the value specified.
	*
	* @see #getPainter()
	*/
	public void setPainter(Painter painter)
	{
		this.painter = painter;
	}

	/**
	* Sets the string describing the shape of the current type to the specified value.   Currently available shapes
	* are: "Ellipse", "Rectangle", "Rounded Rectangle", "Line" and "Dotted Line".
	*
	* @see #getShape()
	*/
	public void setShape(String shape)
	{
		type.setShape(shape);
	}

	/**
	* Sets the textured pattern used to draw the element when it is highlighted to the default pattern.  The default 
	* pattern is  tiny alternating grey and white squares as defined in the static method 
	* <code>GraphicsFactory.getTexturePaint().
	*
	* @see #getTexturePaint()
	*/
	//	public void setTexturePaint()
	//	{
	//		this.texturePaint = defaultTexturePaint;
	//	}

	/**
	* Sets the textured pattern used to draw the element when it is highlighted to the specified value.
	*
	* @see #getTexturePaint()
	*/
	public void setTexturePaint(TexturePaint texturePaint)
	{
		this.texturePaint = texturePaint;
	}

	/**
	* Sets the visual type of the element to the specified value.
	*
	* @see #getType()
	*/
	public void setType(Type type)
	{
		this.type = type;
		setGeneralPath(type.getGeneralPath());
	}

	/**
	* Sets the full-length descriptive name for the current type of the visual element to the specified value.
	*
	* @see #getTypeName()
	*/
	public void setTypeName(String name)
	{
		type.setName(name);
	}

	/**
	* Sets the brief name for the current type of the visual element to the specified value.   This is the name under 
	* which a user-defined type will be saved.
	*
	* @see #getTypeShortName()
	*/
	public void setTypeShortName(String shortName)
	{
		type.setShortName(shortName);
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getElement()
	 */
	public PathwayElement getElement()
	{
		return element;
	}

	/* (non-Javadoc)
	 * @see vp.visual.Renderer#getElement(vp.graph.PathwayElement)
	 */
	public void setElement(PathwayElement element)
	{
		this.element = element;
	}

}
