package vp.visual;

import vp.graph.PathwayElement;
import vp.visual.paint.Painter;
import vp.visual.type.Type;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.TexturePaint;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;

/** 
* The superinterface for all visual graph components.   It is the basis for visual edges, visual vertices and visual
* decorations and contains the methods shared by all three.  It is extended by <code>LabeledVisualGraphElement</code>
* which contains methods shared by vertices and decorations.  The standard implementation is <code>
* AbstractVisualGraphElement</code> from which <code>VisualEdge</code> is inherited.  
* Its structural cognate is <code>GraphElement</code>.
*
* @see edu.yale.graph.GraphElement
* @see edu.yale.graph.visual.LabeledVisualGraphElement
* @see edu.yale.graph.visual.AbstractVisualGraphElement
* @see edu.yale.graph.visual.VisualEdge
* @see edu.yale.graph.visual.VisualVertex
* @see edu.yale.graph.visual.VisualDecoration
*/
public interface Renderer
{
	/** 
	* Returns true if the specified point is contained within the boundaries of the visual element, false otherwise.
	*
	* @param x	The x coordinate of the point.
	* @param y	The y coordinate of the point.
	*/
	boolean containsPoint(int x, int y);

	/**
	* Returns a rectangle expressing the boundaries of the visual element.
	*
	* @see #getBounds2D()
	*/
	Rectangle getBounds();

	/**
	* Returns a 2D rectangle expressing the boundaries of the visual element.
	*
	* @see #getBounds()
	*/
	Rectangle2D getBounds2D();
    
    /**
	* Returns the geometric draw path of the visual element.
	*
	* @see #setGeneralPath(GeneralPath generalPath)
	*/
	GeneralPath getGeneralPath();

	/**
	* Returns the outline color of the visual element.
	*
	* @see #setOutlineColor(Color outlineColor)
	*/
	Color getOutlineColor();

	/**
	* Returns the <code>Painter</code> object used to paint the visual element.
	*
	* @see #setPainter(Painter painter)
	*/
	Painter getPainter();

	/**
	* Returns the textured pattern used to draw the visual element when it is highlighted.
	*
	* @see #setTexturePaint(TexturePaint texturePaint)
	*/
	TexturePaint getTexturePaint();

	/**
	* Returns the visual type of the element.
	*
	* @see #setType(Type type)
	*/
	Type getType();

	/**
	* Notifies the element's painter to paint the element.
	*/
	void paint(Graphics2D g2);

	/**
	* Sets the geometric draw path of the visual element to the value specified.
	* 
	* @see #getGeneralPath()
	*/
	void setGeneralPath(GeneralPath path);

	/**
	* Sets the outline color of the visual element to the value specified.
	*
	* @see #getOutlineColor()
	*/
	void setOutlineColor(Color outlineColor);

	/**
	* Sets the <code>Painter</code> object used by the element to the value specified.
	*
	* @see #getPainter()
	*/
	void setPainter(Painter painter);

	/**
	* Sets the textured pattern used to draw the element when it is highlighted to the specified value.
	*
	* @see #getTexturePaint()
	*/
	void setTexturePaint(TexturePaint tp);

	/**
	* Sets the visual type of the element to the specified value.
	*
	* @see #getType()
	*/
	void setType(Type type);
    
    PathwayElement getElement();
    
    void setElement(PathwayElement element);

}
