package vp.visual.paint;

import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.util.StringTokenizer;

import vp.graph.DummyVertex;
import vp.graph.PathwayElement;
import vp.graph.PathwayVertex;
import vp.util.StringBreaker;

/**
 * The class which paints visual vertices and visual decorations.
 * 
 * @see edu.yale.graph.visual.painting.VisualVertexPainter
 */
public class VertexPainterImp implements VertexPainter
{
    private double tbMargin = 2;
    private double lrMargin = 3;

    public VertexPainterImp()
    {
    }

    /**
     * Paint the component. This calls <code>paintFill</code>,
     * <code>paintOutline</code> and finally <code>paintLabel
     * </code>.
     * 
     * @param vge
     *            The visual element to be painted.
     * @param g2
     *            The graphics context in which to paint.
     */
    public void paint(PathwayElement vge, Graphics2D g2)
    {
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
        paintFill((PathwayVertex) vge, g2);
        paintOutline((PathwayVertex) vge, g2);
        paintLabel((PathwayVertex) vge, g2);
    }

    /**
     * Paint the body of the element.
     */
    public void paintFill(PathwayVertex vge, Graphics2D g2)
    {
        if (vge.getTexturePaint() != null)
            g2.setPaint(vge.getTexturePaint());
        else
            g2.setColor(vge.getFillColor());
        g2.fill(vge.getGeneralPath());
    }

    /**
     * Paint the border of the element.
     */
    public void paintOutline(PathwayVertex vge, Graphics2D g2)
    {
        g2.setColor(vge.getOutlineColor());
        g2.draw(vge.getGeneralPath());
    }

    /**
     * Paint the element's label.
     */
    public void paintLabel(PathwayVertex vge, Graphics2D g2)
    {
        int line = 1;
        int lineHeight = vge.getFontMetrics().getHeight();
        Rectangle bounds = vge.getBounds();
        String broken = StringBreaker.breakString(vge.getLabel());
        StringTokenizer toke = new StringTokenizer(broken, "\n");
        g2.setFont(vge.getFont());
        g2.setColor(vge.getFontColor());
        while (toke.hasMoreTokens())
        {
            g2.drawString(toke.nextToken(),
                (float) (bounds.x + lrMargin), (float) (bounds.y
                    + tbMargin + lineHeight * line - 2));
            line++;
        }
    }

    /**
     * Rescale the element to fit the contents of its label.
     */
    public void rescale(PathwayElement vge)
    {
        double tbm, lrm;
        if ((PathwayVertex) vge instanceof DummyVertex)
        {
            tbm = 0;
            lrm = 0;
        }
        else
        {
            tbm = tbMargin;
            lrm = lrMargin;
        }
        int height = 0;
        int maxWidth = 0;
        int width = 0;
        FontMetrics fontMetrics = ((PathwayVertex) vge)
            .getFontMetrics();
        GeneralPath drawPath = ((PathwayVertex) vge).getGeneralPath();
        AffineTransform transform = new AffineTransform();
        Rectangle origLocation = drawPath.getBounds();
        if (!((PathwayVertex) vge).getLabel().equals("Dummy"))
        {
            int lineHeight = fontMetrics.getHeight();
            String broken = StringBreaker
                .breakString(((PathwayVertex) vge).getLabel());
            StringTokenizer toke = new StringTokenizer(broken, "\n");
            while (toke.hasMoreTokens())
            {
                height += lineHeight;
                width = fontMetrics.stringWidth(toke.nextToken()) + 2;
                maxWidth = width > maxWidth ? width : maxWidth;
            }
            double scalex = (maxWidth + lrm * 2)
                / origLocation.getWidth();
            double scaley = (height + tbm * 2)
                / origLocation.getHeight();
            transform.scale(scalex, scaley);
            drawPath.transform(transform);
        }
        Rectangle newLocation = drawPath.getBounds();
        double transx = origLocation.getMinX()
            - newLocation.getMinX();
        double transy = origLocation.getMinY()
            - newLocation.getMinY();
        int xpos = (vge instanceof PathwayVertex) ? ((PathwayVertex) vge)
            .getXPosition()
            : 0;
        int ypos = (vge instanceof PathwayVertex) ? ((PathwayVertex) vge)
            .getYPosition()
            : 0;
        transx = transx
            - (origLocation.getX() - (xpos - (newLocation.getWidth() / 2)));
        transy = transy
            - (origLocation.getY() - (ypos - (newLocation.getHeight() / 2)));
        transform.setToTranslation(transx, transy);
        drawPath.transform(transform);
    }
}
