package vp.visual.type;

import java.awt.Color;
import java.awt.Font;
import java.awt.geom.GeneralPath;

/**
* An abstract class to define all visual types.   It contains get and set methods used to visualize
* vertices, decorations and edges.   It is implemented generally through <code>TypeImp</code>.   
* User-defined types create an instance of this class.  There are also a handful of pre-defined visual types 
* to be used as defaults.   The <code>InvisibleVertex</code> class is the visual type for <code>DummyVertex</code>.
*
* @see edu.yale.graph.type.TypeImp
* @see edu.yale.graph.type.DefaultVertexType1
* @see edu.yale.graph.type.DefaultVertexType2
* @see edu.yale.graph.type.DefaultEdgeType1
* @see edu.yale.graph.type.DefaultEdgeType2
* @see edu.yale.graph.type.DefaultDecorationType1
* @see edu.yale.graph.type.DefaultDecorationType2
* @see edu.yale.graph.type.InvisibleVertex
*/
public class Type
{
	/**
	* Indicates that this is a vertex type.
	*/
	public static final int VERTEX = 0;

	/**
	* Indicates that this is an edge type.
	*/
	public static final int EDGE = 1;

	/**
	* Indicates that this is a decoration type.
	*/
	public static final int DECORATION = 2;

	/**
	* Designates a solid line shape.
	*/
	public static final String SOLID_LINE = "Solid";

	/**
	* Designates a dotted line shape.
	*/
	public static final String DOTTED_LINE = "Dotted";

	protected Color outlineColor;
	protected GeneralPath drawPath;
	protected Color fillColor;
	protected Font font;
	protected Color fontColor;
	protected String shape;
	protected String name;
	protected String shortName;
	protected boolean hasToolTip;
	protected float curvature;

	public Type()
	{}
	/**
	    * Constructs a new visual type for a vertex or decoration using the specified values.
	    *
	    * @param outlineColor   The color of the element's border.
	    * @param fillColor  The color of the element's body.
	    * @param fontColor  The color of the font used to label the element.
	    * @param font   The font used to label the element.
	    * @param drawPath   The geometric drawPath of the element.
	    * @param shape  A string describing the shape of the element.  This value should be one of: "Ellipse", "Rectangle"
	    * and "Rounded Rectangle".
	    * @param name   A full-length descriptive name for the element.
	    * @param shortName  A brief name for the element.   This is the name under which the type is saved.
	    */
	public Type(
		Color outlineColor,
		Color fillColor,
		Color fontColor,
		Font font,
		GeneralPath drawPath,
		String shape,
		String name,
		String shortName)
	{
		this.outlineColor = outlineColor;
		this.fillColor = fillColor;
		this.fontColor = fontColor;
		this.font = font;
		this.drawPath = drawPath;
		this.shape = shape;
		this.name = name;
		this.shortName = shortName;
	}

	/**
	* Constructs a new visual type for an edge using the specified values.
	*
	* @param outlineColor   The color of the edge.
	* @param shape  A string describing the shape of the edge.  The value should be either "Line" or "Dotted Line".
	* @param name   A full-length descriptive name for the element.
	* @param curvature  The degree of curvature for the edge.  The value should be either 0 for not curved or <>0
	* for curved.
	*/
	public Type(Color outlineColor, String shape, String name)
	{
		this(
			outlineColor,
			null,
			null,
			null,
			new GeneralPath(),
			shape,
			name,
			"");
	}

	/**
	* Returns the degree to which the current type is curved.  The edge is the only element for which this is 
	* applicable.  Currently, there are only two possible values: 0 - not curved and <>0 - curved.
	*
	* @see #setCurvature(float curvature)
	*/
	public float getCurvature()
	{
		return curvature;
	}

	/**
	* Returns the fill color of the current type.   For vertices and decorations it is the color of the body.  
	* Edges do not have a fill color; their color is defined by the outline color.
	*
	* @see #setFillColor(Color fillColor)
	*/
	public Color getFillColor()
	{
		return fillColor;
	}

	/**
	* Returns the font used on the label for the current type.  This is applicable for edges and decorations only.
	*
	* @see #setFont(Font font)
	*/
	public Font getFont()
	{
		return font;
	}

	/**
	* Returns the color of the font used on the label for the current type.   This is applicable for edges and 
	* decorations only.
	*
	* @see #setFontColor(Color fontColor)
	*/
	public Color getFontColor()
	{
		return fontColor;
	}

	/**
	* Returns the geometric draw path for the current type.
	*
	* @see #setGeneralPath(GeneralPath drawPath)
	*/
	public GeneralPath getGeneralPath()
	{
		return drawPath;
	}

	/**
	* Returns a full-length descriptive name for the current type.
	*
	* @see #setName(String name)
	*/
	public String getName()
	{
		return name;
	}

	/**
	* Returns the outline color of the type.   For vertices and decorations it is the color of the border
	* while for edges, it is the color of the entire edge.
	*
	* @see #setOutlineColor(Color outlineColor)
	*/
	public Color getOutlineColor()
	{
		return outlineColor;
	}

	/**
	* Returns a string describing the shape of the current type.  Currently available shapes are: "Ellipse", 
	* "Rectangle", "Rounded Rectangle", "Line" and "Dotted Line".
	*
	* @see #setShape(String shape)
	*/
	public String getShape()
	{
		return shape;
	}

	/**
	* Returns a brief name for the current type.  This is the name under which a user-defined type will be saved.
	*
	* @see #setShortName(String shortName)
	*/
	public String getShortName()
	{
		return shortName;
	}

	/**
	* Returns a new instance of the current type.
	*/
	public Type newInstance()
	{
		return new Type(
			getOutlineColor(),
			getFillColor(),
			getFontColor(),
			getFont(),
			(getGeneralPath() != null)
				? (GeneralPath)getGeneralPath().clone()
				: null,
			getShape(),
			getName(),
			getShortName());
	}

	/**
	* Sets the degree to which the current type is curved to the specified value.  The edge is the only element
	* for which this is applicable.  Currently, there are only two possible values: 0 - not curved and <>0 - curved.
	*
	* @see #getCurvature()
	*/
	public void setCurvature(float curvature)
	{
		this.curvature = curvature;
	}

	/**
	* Sets the fill color fo the current type to the specified value.  For vertices and decorations it is the color of 
	* the body.  Edges do not have a fill color; their color is defined by the outline color.
	*
	* @see #getFillColor()
	*/
	public void setFillColor(Color fillColor)
	{
		this.fillColor = fillColor;
	}

	/**
	* Sets the font used on the label for the current type to the specified value.  This is applicable for edges
	* and decorations only.
	*
	* @see #getFont()
	*/
	public void setFont(Font font)
	{
		this.font = font;
	}

	/**
	* Sets the color of the font used on the label for the current type to the specified value.   This is applicable
	* for edges and decorations only.
	*
	* @see #getFontColor()
	*/
	public void setFontColor(Color fontColor)
	{
		this.fontColor = fontColor;
	}

	/**
	* Sets the geometric draw path for the current type to the specified value.
	*
	* @see #getGeneralPath()
	*/
	public void setGeneralPath(GeneralPath drawPath)
	{
		this.drawPath = drawPath;
	}

	/**
	* Sets the full-length descriptive name for the current type to the specified value.
	*
	* @see #getName()
	*/
	public void setName(String name)
	{
		this.name = name;
	}

	/**
	* Sets the outline color of the type to the value specified.  For vertices and decorations it is the color of the  * border while for edges, it is the color of the entire edge.
	*
	* @see #getOutlineColor()
	*/
	public void setOutlineColor(Color outlineColor)
	{
		this.outlineColor = outlineColor;
	}

	/**
	* Sets the string describing the shape of the current type to the specified value.   Currently available shapes
	* are: "Ellipse", "Rectangle", "Rounded Rectangle", "Line" and "Dotted Line".
	*
	* @see #getShape()
	*/
	public void setShape(String shape)
	{
		this.shape = shape;
	}

	/**
	* Sets the brief name for the current type to the specified value.   This is the name under which a user-defined
	* type will be saved.
	*
	* @see #getShortName()
	*/
	public void setShortName(String shortName)
	{
		this.shortName = shortName;
	}
    
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj)
	{
		if (obj instanceof Type)
		{
			Type t = (Type)obj;
			if ((t.getOutlineColor().equals(outlineColor))
				&& (t.getFillColor() != null
					&& (t.getFillColor().equals(fillColor)))
				&& (t.getFontColor() != null
					&& (t.getFontColor().equals(fontColor)))
				&& (t.getFont() != null
					&& (t.getFont().getName().equals(font.getName()))
					&& (t.getFont().getStyle() == font.getStyle())
					&& (t.getFont().getSize() == font.getSize()))
				&& (t.getShape().equals(shape)))
				return true;
			return false;
		}
		return false;
	}
    
    /* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode()
	{
		return outlineColor.hashCode() / 7
			+ fillColor.hashCode() / 7
			+ fontColor.hashCode() / 7
			+ (3 * font.hashCode()) / 7
			+ shape.hashCode() / 7;
	}

}
